/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995, 1996, 1997, 1998
 *========================================================================
 *              FILE NAME: config.h
 *            DESCRIPTION: compile time configurable options for 
 *								 : XNotesPlus
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/

/* HPUX Installation */
prefix=/opt/XNotesPlus

/*
 * Motif or GTK version?  Only one can be done at a time.  The header
 * files don't allow for more than one at a time.
 *
 * If you use LessTif, make sure you uncomment both DO_MOTIF and
 * DO_LESSTIF.
 *
#define DO_LESSTIF
#define DO_MOTIF
#define DO_GTK
 */
#define DO_GTK


/*
 * Where are the libraries and include files?  You only need to set one of
 * the following two sets of values (either the GTK ones or the Motif ones,
 * but not both).  
 * 
 * Uncomment and set these if your GTK libs and include files are installed 
 * in a nonstandard location.
 * 
 * The USE_GTK12 should be uncommented if you're using the GTK 1.2
 * distribution.  By default, GTK 1.0 is assumed.
 * 
#define NON_STD_GTK
#define USE_GTK12
GTK_LIB=/usr/local/lib/
GTK_INC=/usr/local/include
GLIB_INC=/usr/local/lib/glib/include
 * 
 */
#define NON_STD_GTK
#define USE_GTK12
GTK_LIB=/opt/gtk+/lib/ -L/opt/glib/lib
GTK_INC=/opt/gtk+/include
GLIB_INC=/opt/glib/include -I/opt/glib/lib/glib/include

/*
 * Uncomment and set these if your Motif and/or XPM libs and include files
 * are installed in a nonstandard locations.
 * 
#define NON_STD_XPM
XPM_LIB=/usr/local/lib/lib
XPM_INC=/usr/local/lib/include
 * 
#define NON_STD_MOTIF
MOTIF_LIB=/usr/local/lesstif/lib
MOTIF_INC=/usr/local/lesstif/include
 *
 */

#define NON_STD_XPM
XPM_LIB=/opt/xpm/lib
XPM_INC=/opt/xpm/include

/*
 * Uncomment this is you use Linux or FreeBSD and CDE and get unresolved
 * references to "_Xsetlocal".
 *
#define LINUX_CDE
 */

/*
 * If the above still doesn't work for you, try this one instead.  Only the
 * this one *or* the previous one should be uncommented, not both.
 */
#define NO_XSETLOCALE

/*
 * Uncomment this if you have an older version of the Pilot-Link tools,
 * ie the version of pilot-addresses you have does not have the -a and -e
 * options.  If you don't do this, the envelopes "Load" option may crash
 * XNotesPlus when it tries to read the wrong output format from the
 * pilot-addresses program.
#define OLD_PILOT_ADDR
 */

/*
 * If you use the GTK libraries compiled with X Input enabled then
 * uncomment the following line.  Neither Motif nor LessTif supports this 
 * option.  It doesn't hurt to define this with Motif and LessTif, but it does
 * if your GTK does *not* support USE_XINPUT.
 */
/*#define USE_XINPUT*/

/*
 * Uncomment this next one if you want to use the fancy background pixmaps
 * in the menus and other places.  Its nice if you have high color
 * displays, but don't bother if you have an 8 bit (256 color display).
 *
 * BTW, this option will make use of the XNotesPlus.{ad,rc}.themed application
 * defaults files.  If you want the old version, which uses
 * XNotesPlus.{ad,rc}.standard, then don't define this.
 * 
 * 
 */
#define USE_FANCY_XPM

/*
 * Need a statically linked binary?  Uncomment the following.
 * If you're on Linux (or some other system that uses dlopen()) then you
 * need to uncomment USE_DLLIB too.
#define MAKE_STATIC
#define USE_DLLIB
 */

/*
 * Uncomment the following if you use a system that doesn't know about
 * bzero() - like Solaris, for example.
 */
#define NO_BZERO

/*
 * Uncomment this if you want to turn the debug macros off.  By default
 * the macros expand into real source code.  Note that even if
 * expanded, no debug output will be generated unless you use the appropriate
 * command line options: --debugfile <file> and --debuglevel <level>
 *
 * This isn't really very useful unless you want to seriously reduce the
 * binary size.
#define DEBUGOFF
 */


/*
 * Uncomment this if you use gcc.  If your "cc" is a sym-link to gcc or
 * you've renamed gcc to cc, then uncomment this as well.  You don't
 * have to (I don't think), but it helps.
 */
/*#define HASGCC*/

/*
 * Define the tar command use for building a distributable package here.
 * Almost no one (except people like me who use two different versions of
 * tar) will need to change this.
TAR=gtar cvzf 
 */
TAR=tar cvzf 

/*=========================================================================
 * End of configurable options.
 * You don't need to change anything below this line.
 *=========================================================================*/

/*
 * Do we need to hack around the bzero() problem?
 */
#ifdef NO_BZERO
USE_MEMSET = -DSYSV_USE_MEMSET
#endif

#ifdef NO_XSETLOCALE
LOCAL_XSETLOCALE = -DUSE_LOCAL_XSETLOCALE
#endif

/*
 * Put all those defines together.
 */
OTHER_DEFINES = $(USE_MEMSET) $(LOCAL_XSETLOCALE)


/*
 * Compile and Link options; complex, but thats what we get when we don't
 * compile with the same compiler across platforms.
 * DEGBUG - turns on local debugging library
 * -Wall  - turns on exhaustive checking by gcc compiler
 * -Wno-unused  - turns off notices about unused parameters and variables.
 *		This is useful because the GUI has lots of unused parameters
 *		in callbacks and these are required by only some compilers.
 */

#ifdef DEBUGOFF
#    ifdef HASGCC
CC = gcc
/* DEFINES   = -g -Wall -Wno-unused $(OTHER_DEFINES)
 */
DEFINES   = -g -Wall $(OTHER_DEFINES)
#    else
DEFINES   = $(OTHER_DEFINES)
#    endif
#else
#    ifdef HASGCC
CC = gcc
/* DEFINES   = -g -Wall -Wno-unused -DDEBUG $(OTHER_DEFINES)
 */
DEFINES   = -g -Wall -DDEBUG $(OTHER_DEFINES)
#    else
CCOPTIONS = -Ae
DEFINES   = -DDEBUG $(OTHER_DEFINES)
#    endif
#endif

/*
 * Set up for which version to build.
 */
#ifdef USE_FANCY_XPM
DEFINE_USE_FANCY_XPM = -DUSE_FANCY_XPM
#endif
#ifdef DO_LESSTIF
DEFINE_LESSTIF = -DDO_LESSTIF
#endif
#ifdef OLD_PILOT_ADDR
DEFINE_OLD_PILOT_ADDR = -DOLD_PILOT_ADDR
#endif
#ifdef USE_GTK12
DEFINE_USE_GTK12 = -DUSE_GTK12
#endif
#ifdef DO_GTK
EXTRA_DEFINES = -DDO_GTK $(DEFINE_USE_GTK12) $(DEFINE_OLD_PILOT_ADDR) $(DEFINE_USE_FANCY_XPM)
#else
EXTRA_DEFINES = -DDO_MOTIF $(DEFINE_LESSTIF) $(DEFINE_OLD_PILOT_ADDR) $(DEFINE_USE_FANCY_XPM)
#endif

/*
 * Add X Input library, if necessary.
 */
#ifdef USE_XINPUT
XINPUT = -lXi
#else
XINPUT = 
#endif

/*
 * For the masochistic types, who need a static binary.
 */
#ifdef MAKE_STATIC
EXTRA_LDOPTIONS = -static
#endif
#ifdef USE_DLLIB
DLLIBS = -ldl
#endif
