/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998,1999.
 *========================================================================
 *              FILE NAME: addr_manager.c
 *            DESCRIPTION: manage the Palm Pilot address list
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAddressManager XPUpdateAddrList
 *      PRIVATE FUNCTIONS: CloseAddrMgr UpdateDetails CategoryCB
 *                       : FilterFieldCB UpdateList CallXPEnvelope
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ADDR_MGR_C
#ifndef OLD_PILOT_ADDR

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void	CloseAddrMgr();
static void	UpdateDetails();
static void	FilterFieldCB();
static void	CategoryCB();
static void	UpdateList();
static void	CallXPEnvelope();
static void	RowSelected();
static void	SearchKeyPress();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*Top_Level;

extern PilotAddr_T			*Pilot_Address;
extern PilotAddrCategory_T	*Pilot_Addr_Categories;

extern GdkPixmap	*details_xbm;
extern GdkBitmap	*details_mask;


/* === Static Variables === */
static GtkWidget	*addr_mgr_dialog;
static char *filter_strings[]={
	"None",
	"Last",
	"First",
	"Company",
	"City",
	NULL
};

static int			filter_index;
static int			category_index;
static GtkWidget	*addr_list;
static GtkWidget	*search_text;
static GtkWidget	*details;
static char			*address_array[1];
static int			list_row_selected=-1;

static ListEntries_T		*ListEntries=NULL;


/*========================================================================
 *	Name:			XPAddressManager
 *	Prototype:	void XPAddressManager()
 *					
 *	Description:
 * Setup and present the Address Manager window.
 *
 *	Notes:
 *========================================================================*/
void
XPAddressManager()
{
	char			buf[512];
	int			index;
	int			width;
	

	GtkWidget		*vbox, *vbox2, *close_button; 
	GtkWidget		*addr_viewport;
	GtkWidget		*hbox;
	GtkWidget		*envelope_button; 
	GtkWidget		*category_menu;
	GtkWidget		*filter_menu;
	GtkWidget		*frame1, *frame2, *label;
	GtkWidget		*menu, *menuitem;
	GtkWidget		*eventbox;
	GtkStyle			*style;
	GShell_T			gshell;
	
	PilotAddr_T				*current_link;
	PilotAddrCategory_T	*addr_cat_link;
	ListEntries_T			*current_list_entry, *next_list_entry;

	/*
	 * Grab PalmPilot addresses from text file.
	 */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);
	if ( XPPilotParseAddr(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCDB_FAIL);
		return;
	}

	/*
	 * Free up any existing link lists for the List entries.
	 * We'd have these if we've opened this dialog at least once.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * Allow only one window at a time.
	 */
	if ( addr_mgr_dialog != NULL )
		return;

	/*
	 * The addr dialog parent window.
	 */
	addr_mgr_dialog = 
		XPCreateGenericShell(AddrMgrDialog, ACCEPT_BUTTON|CLOSE_BUTTON,
			&gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (addr_mgr_dialog), "destroy",
			GTK_SIGNAL_FUNC (CloseAddrMgr), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	close_button = gshell.cancel;
	envelope_button = gshell.accept;

	/*
	 * Change the Accept button to say "Envelopes"
	 */
	gtk_label_set(GTK_LABEL(GTK_BIN(envelope_button)->child),XP_S_ENVELOPES);

	/*
	 * An hbox, inside which we stuff the scrolled window and a vbox.
	 */
	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 4);
	gtk_widget_show(hbox);

	/*
	 * A scrolled list of address.
	 */
	addr_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (addr_viewport, "date_formats");
#ifdef USE_GTK12
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (addr_viewport),
		GTK_POLICY_NEVER,
		GTK_POLICY_AUTOMATIC);
#else
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (addr_viewport),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
#endif
	gtk_container_add (GTK_CONTAINER (hbox), addr_viewport);
	gtk_widget_show (addr_viewport);

	addr_list = gtk_clist_new(1);
	gtk_widget_set_name (addr_list, "find_list");
	gtk_container_add (GTK_CONTAINER (addr_viewport), addr_list);
	gtk_widget_show (addr_list);
	gtk_clist_set_selection_mode (GTK_CLIST (addr_list), GTK_SELECTION_BROWSE);

	vbox2 = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 4);
	gtk_widget_show(vbox2);

	/*
	 * Category menu.
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PILOT_CATEGORIES);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 2);
	gtk_widget_show(label);

	addr_cat_link = Pilot_Addr_Categories;
	index=0;
	menu = gtk_menu_new();
	menuitem = gtk_menu_item_new_with_label(XP_S_ALL_ADDR);
	gtk_widget_set_name (menuitem, "option_menus");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
		GTK_SIGNAL_FUNC(CategoryCB), (gpointer) index++);
	while ( addr_cat_link != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(addr_cat_link->name);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(CategoryCB), (gpointer) index);
		index++;
		addr_cat_link = addr_cat_link->next;
	}
	category_menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (category_menu), 0);
	gtk_box_pack_start (GTK_BOX (hbox), category_menu, FALSE, TRUE, 0);
	gtk_widget_show (category_menu);
	category_index=0;
 

	/*
	 * Filter On menu.
	 */
	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PILOT_FILTER);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 2);
	gtk_widget_show(label);

	menu = gtk_menu_new();
	index=0;
	while ( filter_strings[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(filter_strings[index]);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(FilterFieldCB), (gpointer) index);
		index++;
	}
	filter_menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (filter_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (filter_menu), 0);
	gtk_box_pack_start (GTK_BOX (hbox), filter_menu, FALSE, TRUE, 0);
	gtk_widget_show (filter_menu);
	filter_index = 0;


	/*
	 * Search prompt.
	 */
	frame1 = gtk_frame_new(XP_S_PILOT_FILTER_TEXT);
	gtk_frame_set_shadow_type (GTK_FRAME(frame1), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox2), frame1, FALSE, TRUE, 0);
	gtk_widget_show(frame1);

	search_text = gtk_entry_new();
	gtk_widget_set_name (search_text, "find_list");
	gtk_widget_show(search_text);
	gtk_container_add(GTK_CONTAINER(frame1), search_text);
	gtk_widget_set_sensitive ( search_text, FALSE );
   gtk_entry_set_text(GTK_ENTRY(search_text), "");

	/* Allow user to hit enter */
	gtk_signal_connect(GTK_OBJECT(search_text), "key_press_event",
		GTK_SIGNAL_FUNC (SearchKeyPress), (gpointer)NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(search_text), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_SEARCH);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(filter_menu), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_FILTER);


	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		if (	( strlen(current_link->last)  != 0 ) && 
				( strlen(current_link->first) != 0) )
			sprintf(buf, "%s %s", current_link->first, current_link->last);
		else
		if ( strlen(current_link->last) != 0 )
			sprintf(buf, "%s", current_link->last);
		else
		if ( strlen(current_link->first) != 0 )
			sprintf(buf, "%s", current_link->first);
		else
		if ( strlen(current_link->company) != 0 )
			sprintf(buf, "%s", current_link->company);
		else
			sprintf(buf, "Unknown person?");

		address_array[0]=buf;
		gtk_clist_append(GTK_CLIST(addr_list), address_array);
		current_link = current_link->next;
	}


	/*
	 * Details List.  This shows some relevant information about the
	 * selected entry in the list of names.  However, it won't show all the
	 * info.  That only comes when you edit the entry.
	 */
	frame2 = gtk_frame_new(XP_S_PILOT_ADDR_DETAILS);
	gtk_frame_set_shadow_type (GTK_FRAME(frame2), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox2), frame2, FALSE, TRUE, 0);
	gtk_widget_show(frame2);

	/*
	 * Labels don't have backgrounds (their rendered transparent over their 
	 * parent windows).  So we stuff an eventbox in here to add the
	 * background pixmap.
	 */
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(frame2), eventbox);
	gtk_widget_show (eventbox);

	/*
	 * Set the background pixmap for the details window.
	 */
	style = gtk_style_copy( gtk_widget_get_style( eventbox ) );
	style->bg_pixmap[GTK_STATE_NORMAL] = details_xbm;
	gtk_widget_set_style(eventbox, style);

	/*
	 * The details for address entries.
	 */
	details = gtk_label_new(XP_S_DETAILS_DATA);
	gtk_widget_set_name (details, "details");
	gtk_container_add(GTK_CONTAINER(eventbox), details);
	gtk_widget_show(details);

	/*
	 * Populate the list with all entries initially.
	 */
	UpdateList();
	UpdateDetails();

	/*
	 * Callback to use when an address has been selected.
	 */
	gtk_signal_connect (GTK_OBJECT (addr_list), "select_row",
			GTK_SIGNAL_FUNC (RowSelected), NULL);

	/*
	 * Callback to manage envelopes.
	 */
	gtk_signal_connect (GTK_OBJECT (envelope_button), "clicked",
			GTK_SIGNAL_FUNC (CallXPEnvelope), NULL);

	/*
	 * Callback to close the window.
	 */
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (CloseAddrMgr), NULL);

	/*
	 * Display the dialog.
	 */
	gtk_widget_realize(addr_mgr_dialog);
	XPCenterWindow(addr_mgr_dialog, NULL, addr_mgr_dialog);
	gtk_widget_show( addr_mgr_dialog );

	/*
	 * Clear the text - we had dummy text in there just to force the right
	 * width of the frame.
	 */
	gtk_label_set(GTK_LABEL(details),"");

	/*
	 * Resize the address list so we don't need a scrollbar.
	 */
	gdk_window_get_size(addr_list->window, &width, NULL);
	gtk_widget_set_usize(addr_viewport, width+54, -1);

	/*
	 * Set the first entry in the list - if there is one - to be selected
	 * We have to do this after the window is visible.
	 */
	if ( Pilot_Address != NULL )
	{
		list_row_selected = 0;
		gtk_clist_select_row(GTK_CLIST(addr_list), 0, 0);
		gtk_clist_moveto(GTK_CLIST (addr_list), 0, -1, 0.0, 0.0);
	}
	else
		list_row_selected = -1;

	/*
 	 * Don't allow resizing of the window.
	 */
	gtk_window_set_policy(GTK_WINDOW(addr_mgr_dialog), FALSE, FALSE, FALSE);
}


/*========================================================================
 *	Name:			CloseAddrMgr
 *	Prototype:	void CloseAddrMgr()
 *					
 *	Description:
 * Close the Address Manager dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseAddrMgr()
{
	if ( addr_mgr_dialog != NULL )
	{
		gtk_widget_destroy( addr_mgr_dialog );
		addr_mgr_dialog = NULL;
	}

}


/*========================================================================
 *	Name:			XPUpdateAddrList
 *	Prototype:	void XPUpdateAddrList()
 *					
 *	Description:
 * Update the list when requested, like when a new category, filter field,
 * or search text is set.
 *
 *	Notes:
 * This is used for keyboard bindings - do we need it with GTK?
 *========================================================================*/
void
XPUpdateAddrList(
)
{
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			CategoryCB
 *	Prototype:	static void CategoryCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
#ifdef USE_GTK12
static void
CategoryCB(
	int			index
)
#else
static void
CategoryCB(
	GtkWidget	*widget,
	int			index
)
#endif
{
	category_index = index;
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			FilterFieldCB
 *	Prototype:	static void FilterFieldCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
#ifdef USE_GTK12
static void
FilterFieldCB(
	int			index
)
#else
static void
FilterFieldCB(
	GtkWidget	*widget,
	int			index
)
#endif
{
	filter_index = index;
	if ( filter_index == 0 )
	{
   	gtk_entry_set_text(GTK_ENTRY(search_text), "");
		gtk_widget_set_sensitive ( search_text, FALSE );
	}
	else
		gtk_widget_set_sensitive ( search_text, TRUE );

	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			RowSelected
 *	Prototype:	void RowSelected()
 *					
 *	Description:
 * Update the text in the Details box when a new entry is selected.
 *
 *	Notes:
 *========================================================================*/
static void
RowSelected(
	GtkWidget		*clist,
	gint				row,
	gint				column,
	GdkEventButton	*event,
	gpointer			data
)
{
	list_row_selected = row;
	UpdateDetails();
}


/*========================================================================
 *	Name:			UpdateDetails
 *	Prototype:	void UpdateDetails()
 *					
 *	Description:
 * Update the text in the Details box when a new entry is selected.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateDetails()
{
#ifdef DEBUG
	char	*fname="UpdateDetails()";
#endif

	PilotAddr_T			*current_link;
	ListEntries_T		*current_list_entry;
	char					buf[512];
	char					name[512];

	DBGEnter();

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		if ( current_list_entry->id == list_row_selected )
			break;
		current_list_entry = current_list_entry->next;
	}

	/*
	 * Place that links data into a buffer.
	 */
	if ( current_list_entry != NULL )
	{
		current_link = current_list_entry->linkptr;
		if (	(strlen(current_link->first)!=0) || 
				(strlen(current_link->last) !=0) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (strlen(current_link->company)!=0)
			sprintf(name, "%s", current_link->company);
		else
			sprintf(name, "Unknown person?");

		sprintf(buf,
			"%s\n"
			"%s\n%s %s %s\n"
			"%s\n%s\n%s\n%s\n%s",
			name,
			current_link->address, current_link->city, 
			current_link->state, current_link->zip, 
			current_link->home, 
			current_link->work, 
			current_link->fax, 
			current_link->other, 
			current_link->email
		);

		/*
		 * Update the label in the Details box.
		 */
		gtk_label_set(GTK_LABEL(details),buf);
	
		/* Update Envelope window, if its opened */
		XPUpdateToText(NULL, (PilotAddr_T *)current_link, NULL);
		XPFillEmailList((PilotAddr_T *)current_link);
	}
	else
	{
		sprintf(buf, XP_S_NO_ADDRS);
		gtk_label_set(GTK_LABEL(details),buf);
	}
}


/*========================================================================
 *	Name:			CallXPEnvelope
 *	Prototype:	void CallXPEnvelope()
 *					
 *	Description:
 * Grab the selected link and pass it to the Envelope dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CallXPEnvelope(
	GtkWidget		*widget,
	int				data
)
{
	ListEntries_T		*current_list_entry;

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	if ( list_row_selected != -1 )
	{
		current_list_entry = ListEntries;
		while (current_list_entry != NULL)
		{
			if ( current_list_entry->id == list_row_selected )
				break;
			current_list_entry = current_list_entry->next;
		}
		XPEnvelope((PilotAddr_T *)current_list_entry->linkptr);
	}
	else
		XPEnvelope((PilotAddr_T *)NULL);
}



/*========================================================================
 *	Name:			SearchKeyPress
 *	Prototype:	void SearchKeyPress()
 *					
 *	Description:
 * Check for ENTER key in text entry fields.
 *
 *	Notes:
 *========================================================================*/
static void
SearchKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			UpdateList();
			UpdateDetails();
			break;
		default:
			break;
	}
}


/*========================================================================
 *	Name:			UpdateList
 *	Prototype:	void UpdateList()
 *					
 *	Description:
 * Update the scrolled list of names.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateList(
	GtkWidget		*widget,
	int				data
)
{
	gboolean					found1, found2;
	char						buf[512];
	PilotAddr_T				*current_link;
	PilotAddrCategory_T	*cat_link;
	ListEntries_T			*current_list_entry=NULL;
	ListEntries_T			*next_list_entry=NULL;
	int						count;
	char						*search_string, *fielddata;
	int						num_entries;

	/*
	 * Clear the address list of all items.
	 */
	gtk_clist_freeze(GTK_CLIST(addr_list));
	gtk_clist_clear(GTK_CLIST(addr_list));
	num_entries = -1;

	/*
	 * Free up current link list of List entries.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * Grab the current list options: category, filter field, filter string.
	 * Convert it to lowercase to make searching case insensitive.
	 */
	search_string = gtk_entry_get_text(GTK_ENTRY(search_text));
	XPConvertCase(search_string, XP_LOWER, 0);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		found1 = found2 = False;
		if ( strlen(search_string) > 0 )
		{
			switch (filter_index)
			{
				case 0:
					fielddata = NULL;
					break;
				case 1:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->last)+1);
					strcpy(fielddata, current_link->last);
					break;
				case 2:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->first)+1);
					strcpy(fielddata, current_link->first);
					break;
				case 3:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->company)+1);
					strcpy(fielddata, current_link->company);
					break;
				default:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->city)+1);
					strcpy(fielddata, current_link->city);
					break;
			}

			/*
			 * If the search string is not empty and this entry contains the
			 * string in the field specified, flag the entry.
			 */
			if ( fielddata != NULL )
			{
				XPConvertCase(fielddata, XP_LOWER, 0);
				if ( strstr(fielddata, search_string) != NULL )
					found1 = True;
				free(fielddata);
				fielddata = NULL;
			}
			else
				found1 = True;
		}
		else
			found1 = True;

		/*
		 * Now check if the category matches.
		 */
		if ( category_index != 0 )
		{
			cat_link = Pilot_Addr_Categories;
			count=1;
			while ( (cat_link != NULL ) && ( count != category_index ) )
			{
				cat_link = cat_link->next;
				count++;
			}
			if ( cat_link != NULL )
			{
				if ( strcmp(current_link->category, cat_link->name) == 0 )
				{
					found2 = True;
				}
			}
		}
		else
			found2 = True;

		/*
		 * If we found matches in both cases (or can ignore them) then we
		 * have a match.  Grab the appropriate fields to add to the list.
		 */
		if ( found1 && found2 )
		{
			if (	( strlen(current_link->last)  != 0 ) && 
					( strlen(current_link->first) != 0) )
				sprintf(buf, "%s %s", current_link->first, current_link->last);
			else
			if ( strlen(current_link->last) != 0 )
				sprintf(buf, "%s", current_link->last);
			else
			if ( strlen(current_link->first) != 0 )
				sprintf(buf, "%s", current_link->first);
			else
			if ( strlen(current_link->company) != 0 )
				sprintf(buf, "%s", current_link->company);
			else
				sprintf(buf, "Unknown person?");

			address_array[0]=buf;
			gtk_clist_append(GTK_CLIST(addr_list), address_array);
			num_entries++;

			/*
			 * Add entry to the link list that tracks whats visible.
			 */
			if ( ListEntries == NULL )
			{
				ListEntries = (ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				ListEntries->id = num_entries;
				ListEntries->linkptr = current_link;
				ListEntries->next = NULL;
				current_list_entry = ListEntries;
			}
			else
			{
				current_list_entry->next = 
					(ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				current_list_entry = current_list_entry->next;
				current_list_entry->id = num_entries;
				current_list_entry->linkptr = current_link;
				current_list_entry->next = NULL;
			}
		}

		current_link = current_link->next;
	}

	if (num_entries > 0 )
	{
		list_row_selected = 0;
		gtk_clist_select_row(GTK_CLIST(addr_list), 0, 0);
		gtk_clist_moveto(GTK_CLIST (addr_list), 0, -1, 0.0, 0.0);
	}

	gtk_clist_thaw(GTK_CLIST(addr_list));
}
#endif
