/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: email.c
 *            DESCRIPTION: email routines for XNotesPlus notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateEmailNotePrompt
 *      PRIVATE FUNCTIONS: EmailPopDown, EmailNote
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_EMAIL_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>


/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void EmailPopDown();
static void EmailNote();
static void EmailEntrySelected();
static void	AKeyPress();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;


/* === Static Variables === */
static GtkWidget		*email_widget=NULL;
static GtkWidget		*email_text;
static GtkWidget		*email_viewport;
static GtkWidget		*email_list;
static GtkWidget		*email_name;



/*========================================================================
 *	Name:			XPCreateEmailNotePrompt
 *	Prototype:	XPCreateEmailNotePrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a window in which the email address can be entered.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
#ifdef USE_GTK12
void
XPCreateEmailNotePrompt(
	int			index
)
#else
void
XPCreateEmailNotePrompt(
	GtkWidget	*widget,
	int			index
)
#endif
{
#ifdef DEBUG
	char	*fname="XPCreateEmailNotePrompt()";
#endif

	StickyNote		*pn;
	GtkWidget		*email_label, *email_cancel, *email_accept;
	GtkWidget		*vbox;
	GtkWidget		*eventbox;
	GShell_T			gshell;

	DBGEnter();

	/*
	 * Only one copy at at time!
	 */
	if ( email_widget != NULL )
		return;

	/*
	 * Find the note we're opening.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	email_widget = XPCreateGenericShell(NotesEmailPrompt,
			ACCEPT_BUTTON | CANCEL_BUTTON, &gshell );

	gtk_signal_connect (GTK_OBJECT (email_widget), "destroy",
			GTK_SIGNAL_FUNC (EmailPopDown), NULL);

	vbox = gshell.vbox;
	email_cancel = gshell.cancel;
	email_accept = gshell.accept;


	/* the label window */
	email_label = gtk_label_new(XP_S_EMAIL);
	
	gtk_box_pack_start (GTK_BOX (vbox), email_label, TRUE, TRUE, 0);
	gtk_widget_show(email_label);

	/*
	 * The text entry area for the email address.
	 */
	email_text = gtk_entry_new();
	gtk_widget_set_name (email_text, "addresses");
	gtk_box_pack_start (GTK_BOX (vbox), email_text, TRUE, TRUE, 0);
	gtk_widget_show(email_text);
   gtk_entry_set_text(GTK_ENTRY(email_text), app_res.email_addr);

	/* Allow user to hit enter */
	gtk_signal_connect(GTK_OBJECT(email_text), "key_press_event",
		GTK_SIGNAL_FUNC (AKeyPress), (gpointer)pn);

#ifndef OLD_PILOT_ADDR

	/*
	 * The list of possible email addresses provided by the Address Manager.
	 */
	email_name = gtk_label_new(XP_S_EMAIL_NONAME);
	gtk_box_pack_start (GTK_BOX (vbox), email_name, TRUE, TRUE, 0);
	gtk_widget_show(email_name);
	
	eventbox = gtk_event_box_new();
	gtk_container_add (GTK_CONTAINER (vbox), eventbox);
	gtk_widget_show (eventbox);

	email_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (email_viewport, "addresses");
#ifdef USE_GTK12
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (email_viewport),
		GTK_POLICY_NEVER,
		GTK_POLICY_AUTOMATIC);
#else
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (email_viewport),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
#endif
	gtk_container_add (GTK_CONTAINER (eventbox), email_viewport);
	gtk_widget_show (email_viewport);

	email_list = gtk_clist_new(1);
	gtk_widget_set_name (email_list, "addresses");
	gtk_container_add (GTK_CONTAINER (email_viewport), email_list);
	gtk_widget_show (email_list);
	gtk_clist_set_selection_mode (GTK_CLIST (email_list), GTK_SELECTION_SINGLE);

	gtk_signal_connect (GTK_OBJECT (email_list), "select_row",
			GTK_SIGNAL_FUNC (EmailEntrySelected), (gpointer)NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(eventbox), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_EMAIL_LIST);

#endif

	/* 
	 * The Accept and Cancel buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (email_accept), "clicked",
			GTK_SIGNAL_FUNC (EmailNote), (gpointer)pn);

	gtk_signal_connect (GTK_OBJECT (email_cancel), "clicked",
			GTK_SIGNAL_FUNC (EmailPopDown), (gpointer)NULL);

	/*
	 * Prevent resizing.
	 */
	gtk_window_set_policy(GTK_WINDOW(email_widget), FALSE, FALSE, FALSE);

	/*
	 * Display the window.
	 */
	gtk_widget_realize(email_widget);
	XPCenterWindow(email_widget, NULL, email_widget);
	gtk_widget_show( email_widget );
}


/*========================================================================
 *	Name:			EmailPopDown
 *	Prototype:	EmailPopDown(GtkWidget *widget, gpointer data)
 *					
 *	Description:
 * Close the Email window
 *========================================================================*/
static void
EmailPopDown(
	GtkWidget	*widget,
	gpointer		data
)
{
	if ( email_widget != NULL )
	{
		gtk_widget_destroy(email_widget);
		email_widget = NULL;
	}
}


/*========================================================================
 *	Name:			AKeyPress
 *	Prototype:	void AKeyPress()
 *					
 *	Description:
 * Check for ENTER key in text entry fields.
 *
 *	Notes:
 *========================================================================*/
static void
AKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event,
	StickyNote	pn
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			EmailNote(NULL, pn);
			break;
		default:
			break;
	}
}



/*========================================================================
 *	Name:			EmailNote
 *	Prototype:	EmailNote(GtkWidget *widget, StickyNote *pn)
 *					
 *	Description:
 * Email a notes text to the specified recipient.
 *========================================================================*/
static void
EmailNote(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			syscmd[1024];
	char			emailcmd[1024];
	char			*addressee;

	/*
	 * Don't email blank notes.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp("No text in note to email!");
		return;
	}

	/*
	 * Save the text to a temporary file
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(syscmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(syscmd);
		return;
	}

	len = strlen(pn->pn_text);
	fwrite(pn->pn_text, sizeof(char), len, fp);
	fclose(fp);

	/*
	 * Use the emailcmd resource to determine how to email the note
	 */
	addressee = gtk_entry_get_text(GTK_ENTRY(email_text));
	sprintf (emailcmd, app_res.email_cmd, pn->pn_name->str, addressee);
	sprintf (syscmd, "cat %s | %s", tmpfile, emailcmd );
	system ( syscmd );

	/*
	 * clean up the temporary file
	 */
	sprintf( syscmd, "rm %s", tmpfile );
	system ( syscmd );

	EmailPopDown(NULL, NULL);
}

#ifndef OLD_PILOT_ADDR

/*========================================================================
 *	Name:			EmailEntrySelected
 *	Prototype:	EmailEntrySelected()
 *					
 *	Description:
 * Email address selected by user from scrolled list.
 *========================================================================*/
static void
EmailEntrySelected(
	GtkWidget		*clist,
	gint				row,
	gint				column,
	GdkEventButton	*event,
	gpointer			data
)
{
	char	*choice;

	gtk_clist_get_text(GTK_CLIST(email_list), row, 0, &choice);
	gtk_entry_set_text(GTK_ENTRY(email_text), choice);
}


/*========================================================================
 *	Name:			XPFillEmailList
 *	Prototype:	XPFillEmailList(PilotAddr_T *current_link)
 *					
 *	Description:
 * Populate the list of email addresses.
 *========================================================================*/
void
XPFillEmailList(
	PilotAddr_T		*current_link
)
{
#ifdef DEBUG
	char *fname="XPFillEmailList()";
#endif
	char				*buf, *bufptr;
	char				*entry = NULL;
	char				name[128];
	int				count=0;
	char				*email_array[1];

	DBGEnter();

	/* Make sure the envelope dialog is available. */
	if ( email_widget == NULL )
		return;

	/*
	 * Remove any old entries.
	 */
	gtk_clist_freeze(GTK_CLIST(email_list));
	gtk_clist_clear(GTK_CLIST(email_list));

	/*
	 * Build the name and stuff it in the label.
	 */
	if ( ( current_link->last != NULL ) && ( current_link->first != NULL ) )
	{
		if (	( strlen(current_link->last) != 0 ) && 
				( strlen(current_link->first) != 0 ) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (	( strlen(current_link->first) == 0 ) && 
				( strlen(current_link->last) != 0 ) )
			sprintf(name, "%s", current_link->last);
		else
			sprintf(name, "%s", current_link->first);
	}
	else
	if ( current_link->last != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->first != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->company != NULL )
		sprintf(name, "%s", current_link->company);

	DBGPrintf(DBG_INFO, ("name: *%s*\n", name));

#ifdef USE_GTK12
	gtk_label_set_text(GTK_LABEL(email_name), name);
#else
	gtk_label_set(GTK_LABEL(email_name), name);
#endif

	/*
	 * Pull out any email addresses.  Add them, with the name
	 * in parenthesis, if possible, to the ScrolledList window.
	 */

	/* Work field */
	if ( strstr(current_link->work, "@") != NULL )
	{
		if ( strstr(current_link->work, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->work)+1);
			strcpy(buf, current_link->work);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->work)+1);
			strcpy(entry, current_link->work);
		}
		email_array[0]=entry;
		gtk_clist_append(GTK_CLIST(email_list), email_array);
		free(entry);
		count++;
	}

	/* Home field */
	if ( strstr(current_link->home, "@") != NULL )
	{
		if ( strstr(current_link->home, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->home)+1);
			strcpy(buf, current_link->home);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->home)+1);
			strcpy(entry, current_link->home);
		}
		email_array[0]=entry;
		gtk_clist_append(GTK_CLIST(email_list), email_array);
		free(entry);
		count++;
	}

	/* Fax field */
	if ( strstr(current_link->fax, "@") != NULL )
	{
		if ( strstr(current_link->fax, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->fax)+1);
			strcpy(buf, current_link->fax);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->fax)+1);
			strcpy(entry, current_link->fax);
		}
		email_array[0]=entry;
		gtk_clist_append(GTK_CLIST(email_list), email_array);
		free(entry);
		count++;
	}

	/* Email field */
	if ( strstr(current_link->email, "@") != NULL )
	{
		if ( strstr(current_link->email, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->email)+1);
			strcpy(buf, current_link->email);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->email)+1);
			strcpy(entry, current_link->email);
		}
		email_array[0]=entry;
		gtk_clist_append(GTK_CLIST(email_list), email_array);
		free(entry);
		count++;
	}

	/* Other field */
	if ( strstr(current_link->other, "@") != NULL )
	{
		if ( strstr(current_link->other, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->other)+1);
			strcpy(buf, current_link->other);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->other)+1);
			strcpy(entry, current_link->other);
		}
		email_array[0]=entry;
		gtk_clist_append(GTK_CLIST(email_list), email_array);
		free(entry);
		count++;
	}
	gtk_clist_thaw(GTK_CLIST(email_list));

	/*
	 * Set the first entry, if any.
	 */
	if ( count > 0 )
	{
		gtk_clist_get_text(GTK_CLIST(email_list), 0, 0, &entry);
		gtk_entry_set_text(GTK_ENTRY(email_text), entry);
	}
	else
	{
		gtk_entry_set_text(GTK_ENTRY(email_text), "");
	}
}
#endif
