/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: inserts.c
 *            DESCRIPTION: routines that insert simple text data into notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPInsertCalendar, XPInsertDates
 *      PRIVATE FUNCTIONS: CreateInsertPrompt, InsertAccept, InsertPopDown,
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_INSERTS_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void InsertPopDown();
static void InsertAccept();
static void CreateInsertPrompt();
static void GetTime();
static void SetFormat();
static void ToggleGMT();
static void ToggleAbbr();
static void FormatSelected();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;


/* === Static Variables === */
static GtkWidget 		*insert_widget=NULL;
static GtkWidget		*insert_buttons[4];
static GtkWidget		*formats_list;
static int				active_item=1;
static int				saved_item=-1;

static char			dayname[12];
static char			year[5];
static char			monthname[12];
static int			daynum;
static int			wdnum;
static int			monthnum;
static int			yearnum;
static int			hournum;
static int			minnum;
static gboolean	use_gmtime=FALSE;
static gboolean	use_abbr=FALSE;
static char			format[32];
static char			*format_array[1];
static guint		signal_id;



/*========================================================================
 *	Name:			XPInsertCalendar
 *	Prototype:	void XPInsertCalendar(GtkWidget *widget, int index)
 *					
 *	Description:
 * Insert this months calendar into the note
 *
 *	Notes:
 *========================================================================*/
#ifdef USE_GTK12
void
XPInsertCalendar(
	int			index
)
#else
void
XPInsertCalendar(
	GtkWidget	*widget,
	int			index
)
#endif
{
#ifdef DEBUG
	char	*fname="XPInsertCalendar()";
#endif

	StickyNote	*pn;
	char			tmpfile[512];
	char			cmd[1024];
	int			rc;

	DBGEnter();

	/*
	 * Find the note we're adding the calender to.
	 */
	DBGPrintf(DBG_INFO,("savewidget index: %d\n", index));
	if ((pn = XPFindNote(index)) == NULL)
		return;
	DBGPrintf(DBG_INFO,("pn: 0x%08x\n", (int)pn));
	DBGPrintf(DBG_INFO,("savewidget(0): 0x%08x\n", pn->pn_savewidget));

	/*
	 * Run the calendar generating program to get the text.
	 * We use system() instead of popen() because the latter
	 * doesn't reliably tell us if the process actually ran.
	 * system() will provide the return value of the invoked
	 * program.
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	sprintf (cmd, "%s > %s", app_res.calendar_cmd, tmpfile );
	if ( (rc=system ( cmd )) != 0 ) 
	{

		sprintf(cmd, XP_S_CANT_RUN_CMD, app_res.calendar_cmd, rc);
		XPErrPopUp(cmd);

		/* cleanup, if necessary */
		sprintf (cmd, "rm %s", tmpfile );
		system ( cmd );
		return;
	}

	/* stuff the output into the note */
	DBGPrintf(DBG_INFO,("savewidget(1): 0x%08x\n", pn->pn_savewidget));
	XPInsertText(pn, tmpfile, TRUE, FILE_TYPE);

	/*
	 * Changes occured to the note - set its save widget sensitive.
	 */
	DBGPrintf(DBG_INFO,("savewidget(2): 0x%08x\n", pn->pn_savewidget));
	XPSetSaveSensitive(NULL, NULL, pn);
}



/*========================================================================
 *	Name:			XPInsertDates
 *	Prototype:	void XPInsertDates(GtkWidget *widget, int index)
 *					
 *	Description:
 * Callback for inserting a formatted date into the note.
 *
 *	Notes:
 *========================================================================*/
#ifdef USE_GTK12
void
XPInsertDates(
	int			index
)
#else
void
XPInsertDates(
	GtkWidget	*widget,
	int			index
)
#endif
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	CreateInsertPrompt(pn);

}


/*========================================================================
 *	Name:			CreateInsertPrompt
 *	Prototype:	static void CreateInsertPrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a prompt for insert information
 *
 *	Notes:
 *========================================================================*/
static void
CreateInsertPrompt(
	StickyNote	*pn
)
{
	GtkWidget		*formats_viewport;
	GtkWidget		*eventbox;
	GtkWidget		*vbox;
	int				i;
	GShell_T			gshell;

	/*
	 * Prevent multiple windows from being opened at once.
	 */
	if ( insert_widget != NULL )
		InsertPopDown();

	/* initialize marker */
	active_item=0;

	/*
	 * Create a popup shell widget
	 */
	insert_widget = XPCreateGenericShell(NotesInsertPrompt,
			ACCEPT_BUTTON | CANCEL_BUTTON | APPEND_BUTTON | OVERWRITE_BUTTON,
			&gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (insert_widget), "destroy",
			GTK_SIGNAL_FUNC (InsertPopDown), NULL);

	vbox = gshell.vbox;
	insert_buttons[0] = gshell.accept;
	insert_buttons[1] = gshell.cancel;
	insert_buttons[2] = gshell.append;
	insert_buttons[3] = gshell.overwrite;

	/*
	 * Change the labels in the append and overwrite buttons.
	 */
	if (use_gmtime)
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[2])->child),XP_S_LOCAL);
	else
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[2])->child),XP_S_GMT);

	if (use_abbr)
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[3])->child),XP_S_FULL);
	else
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[3])->child),XP_S_ABBR);

	/*
	 * An event box for managing tooltips.
	 */
	eventbox = gtk_event_box_new();
	gtk_box_pack_start (GTK_BOX (vbox), eventbox, TRUE, TRUE, 0);
	gtk_widget_show (eventbox);

	/*
	 * A scrolled list of time/date formats.
	 */
	formats_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (formats_viewport, "date_formats");
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (formats_viewport),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (eventbox), formats_viewport);
	gtk_widget_show (formats_viewport);

	formats_list = gtk_clist_new(1);
	gtk_widget_set_name (formats_list, "find_list");
	gtk_container_add (GTK_CONTAINER (formats_viewport), formats_list);
	gtk_widget_show (formats_list);
	gtk_clist_set_selection_mode (GTK_CLIST (formats_list), GTK_SELECTION_BROWSE);

	/* Grab the current time and stuff it into some local variables */
	GetTime();

	/*
	 * The set of button/labels that provide the various date formats
	 * to use.
	 */
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		format_array[0]=format;
		gtk_clist_append(GTK_CLIST(formats_list), format_array);
	}
	if (saved_item != -1 )
		active_item = saved_item;
	else
	{
		saved_item = 0;
		active_item = 0;
	}

	/* The list item selection callback */
	signal_id = gtk_signal_connect (GTK_OBJECT (formats_list), "select_row",
		GTK_SIGNAL_FUNC (FormatSelected), (gpointer)NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(eventbox), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_INSERT_FORMAT);


	/*
	 * the Cancel/Accept command buttons
	 */
	gtk_signal_connect (GTK_OBJECT (insert_buttons[0]), "clicked",
			GTK_SIGNAL_FUNC (InsertAccept), (gpointer)pn);

	gtk_signal_connect (GTK_OBJECT (insert_buttons[1]), "clicked",
			GTK_SIGNAL_FUNC (InsertPopDown), (gpointer)NULL);

	gtk_signal_connect (GTK_OBJECT (insert_buttons[2]), "clicked",
			GTK_SIGNAL_FUNC (ToggleGMT), (gpointer)NULL);

	gtk_signal_connect (GTK_OBJECT (insert_buttons[3]), "clicked",
			GTK_SIGNAL_FUNC (ToggleAbbr), (gpointer)NULL);

	/*
	 * we need to resize the viewport so we can see more than 1 or 2
	 * formats.
	 */
	gtk_widget_set_usize (GTK_WIDGET (formats_viewport), 
		XP_FIND_WIDTH, XP_FIND_HEIGHT);

	/*
	 * Pop up the window.
	 */
	gtk_widget_realize(insert_widget);
	XPCenterWindow(insert_widget, pn->pn_shellwidget, insert_widget);
	gtk_widget_show( insert_widget );

	/*
	 * Set and move to the current item in the list.  We have to do this
	 * *after* showing the dialog window because the row height for each
	 * entry doesn't get set till then, and the moveto() function needs this
	 * to work properly.
	 */
	gtk_clist_select_row(GTK_CLIST(formats_list), saved_item, 0);
	gtk_clist_moveto(GTK_CLIST (formats_list), saved_item, -1, 0.0, 0.0);

	/*
 	 * Don't allow resizing of the window.
	 */
	gtk_window_set_policy(GTK_WINDOW(insert_widget), FALSE, FALSE, FALSE);
}


/*========================================================================
 *	Name:			InsertAccept
 *	Prototype:	void InsertAccept(GtkWidget *widget, int index)
 *					
 *	Description:
 * Accept selected date value. This routine has the potential to be 
 * very OS specific.
 *
 *	Notes:
 *========================================================================*/
void
InsertAccept(
	GtkWidget	*widget,
	StickyNote 	*pn
)
{
#ifdef DEBUG
	char	*fname="InsertAccept()";
#endif

	DBGEnter();

	GetTime();
	SetFormat();

	/* stuff the output into the note */
	XPInsertText(pn, format, FALSE, TEXT_TYPE);

	gtk_widget_set_sensitive ( pn->pn_savewidget, TRUE );
	InsertPopDown();
}


/*========================================================================
 * InsertPopDown - removes the Insert Dialog box
 *========================================================================*/
static void
InsertPopDown()
{
	if ( insert_widget != NULL )
	{
		gtk_widget_destroy( insert_widget );
		insert_widget = NULL;
	}
}



/*========================================================================
 *	Name:			GetTime 
 *	Prototype:	void GetTime()
 *					
 *	Description:
 *	Grab the current date fields for display in the dialog.
 *
 *	Notes:
 *========================================================================*/
static void
GetTime()
{
	time_t		time_tp;
	struct tm	*tm_tp;

	/* get todays date and current time */
	time(&time_tp);
	if ( use_gmtime )
		tm_tp = gmtime(&time_tp);
	else
		tm_tp = localtime(&time_tp);

	/*
	 * increment the numeric month, since localtime()/gmtime()
	 * return the month from 0 to 11.
	 */
	tm_tp->tm_mon++;

	/*
	 * Save integer values.
	 */
	daynum   = tm_tp->tm_mday;
	monthnum = tm_tp->tm_mon;
	yearnum  = tm_tp->tm_year;
	hournum  = tm_tp->tm_hour;
	minnum   = tm_tp->tm_min;
	wdnum    = tm_tp->tm_wday;
}


/*========================================================================
 *	Name:			SetFormat() 
 *	Prototype:	void SetFormat()
 *					
 *	Description:
 *	Format the current time into the format buffer.
 *
 *	Notes:
 *========================================================================*/
static void
SetFormat()
{
	/*
	 * convert numeric values to strings now, even if we don't use them,
	 * since we need these in more than one place.
	 */
	if (use_abbr)
	{
		switch ( wdnum )
		{
			case 0:	sprintf(dayname, "Sun"); break;
			case 1:	sprintf(dayname, "Mon"); break;
			case 2:	sprintf(dayname, "Tue"); break;
			case 3:	sprintf(dayname, "Wed"); break;
			case 4:	sprintf(dayname, "Thu"); break;
			case 5:	sprintf(dayname, "Fri"); break;
			case 6:	sprintf(dayname, "Sat"); break;
		}
		switch ( monthnum )
		{
			case  0: sprintf(monthname, "Jan"); break;
			case  1: sprintf(monthname, "Feb"); break;
			case  2: sprintf(monthname, "Mar"); break;
			case  3: sprintf(monthname, "Apr"); break;
			case  4: sprintf(monthname, "May"); break;
			case  5: sprintf(monthname, "Jun"); break;
			case  6: sprintf(monthname, "Jul"); break;
			case  7: sprintf(monthname, "Aug"); break;
			case  8: sprintf(monthname, "Sep"); break;
			case  9: sprintf(monthname, "Oct"); break;
			case 10: sprintf(monthname, "Nov"); break;
			case 11: sprintf(monthname, "Dec"); break;
		}

	}
	else
	{
		switch ( wdnum )
		{
			case 0:	sprintf(dayname, "Sunday"); break;
			case 1:	sprintf(dayname, "Monday"); break;
			case 2:	sprintf(dayname, "Tuesday"); break;
			case 3:	sprintf(dayname, "Wednesday"); break;
			case 4:	sprintf(dayname, "Thursday"); break;
			case 5:	sprintf(dayname, "Friday"); break;
			case 6:	sprintf(dayname, "Saturday"); break;
		}
		switch ( monthnum )
		{
			case  0: sprintf(monthname, "January"); break;
			case  1: sprintf(monthname, "February"); break;
			case  2: sprintf(monthname, "March"); break;
			case  3: sprintf(monthname, "April"); break;
			case  4: sprintf(monthname, "May"); break;
			case  5: sprintf(monthname, "June"); break;
			case  6: sprintf(monthname, "July"); break;
			case  7: sprintf(monthname, "August"); break;
			case  8: sprintf(monthname, "September"); break;
			case  9: sprintf(monthname, "October"); break;
			case 10: sprintf(monthname, "November"); break;
			case 11: sprintf(monthname, "December"); break;
		}

	}

	if ( yearnum > 100 )
		sprintf(year,"20");
	else
		sprintf(year,"19");

	switch (active_item)
	{
		case 0:
			sprintf (format, 
				"%02d/%02d/%02d", monthnum, daynum, yearnum );
			break;

		case 1:
			sprintf (format, 
				"<%02d/%02d/%02d>", monthnum, daynum, yearnum );
			break;

		case 2:
			sprintf (format, 
				"(%02d/%02d/%02d)", monthnum, daynum, yearnum );
			break;

		case 3:
			sprintf (format, 
				"%02d/%02d/%02d - %02d:%02d", 
				monthnum, daynum, yearnum, hournum, minnum );
			break;

		case 4:
			sprintf (format, 
				"%02d/%02d/%02d(%02d:%02d)", 
				monthnum, daynum, yearnum, hournum, minnum );
			break;

		case 5:
			sprintf (format, 
				"%s %d, %s%d", monthname, daynum, year, yearnum);
			break;

		case 6:
			sprintf (format, 
				"%s %d, %s%d - %02d:%02d", 
				monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 7:
			sprintf (format, 
				"%s %d, %s%d (%02d:%02d)", 
				monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 8:
			sprintf (format, 
				"%s %s %d, %s%d", 
				dayname, monthname, daynum, year, yearnum);
			break;

		case 9:
			sprintf (format, 
				"%s %s %d, %s%d - %02d:%02d", 
				dayname, monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 10:
			sprintf (format, 
				"%s %s %d, %s%d (%02d:%02d)", 
				dayname, monthname, daynum, year, yearnum, hournum, minnum );
			break;

		/* International formats */
		case 11:
			sprintf (format, 
				"%02d/%02d/%02d", daynum, monthnum, yearnum );
			break;

		case 12:
			sprintf (format, 
				"<%02d/%02d/%02d>", daynum, monthnum, yearnum );
			break;

		case 13:
			sprintf (format, 
				"(%02d/%02d/%02d)", daynum, monthnum, yearnum );
			break;

		case 14:
			sprintf (format, 
				"%02d/%02d/%02d - %02d:%02d", 
				daynum, monthnum, yearnum, hournum, minnum );
			break;

		case 15:
			sprintf (format, 
				"%02d/%02d/%02d(%02d:%02d)", 
				daynum, monthnum, yearnum, hournum, minnum );
			break;

		case 16:
			sprintf (format, 
				"%d %s, %s%d", daynum, monthname, year, yearnum);
			break;

		case 17:
			sprintf (format, 
				"%d %s, %s%d - %02d:%02d", 
				daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 18:
			sprintf (format, 
				"%d %s, %s%d (%02d:%02d)", 
				daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 19:
			sprintf (format, 
				"%s %d %s, %s%d", 
				dayname, daynum, monthname, year, yearnum);
			break;

		case 20:
			sprintf (format, 
				"%s %d %s, %s%d - %02d:%02d", 
				dayname, daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 21:
			sprintf (format, 
				"%s %d %s, %s%d (%02d:%02d)", 
				dayname, daynum, monthname, year, yearnum, hournum, minnum );
			break;
	}
}


/*========================================================================
 *	Name:			ToggleGMT 
 *	Prototype:	void ToggleGMT
 *					
 *	Description:
 * Handle a left arrow click.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleGMT(
	GtkWidget	*widget,
	StickyNote 	*pn
)
{
	int			i;

	if (use_gmtime)
	{
		use_gmtime = FALSE;
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[2])->child),XP_S_GMT);
	}
	else
	{
		use_gmtime = TRUE;
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[2])->child),XP_S_LOCAL);
	}

	gtk_signal_handler_block (GTK_OBJECT (formats_list), signal_id);
	gtk_clist_freeze(GTK_CLIST(formats_list));
	gtk_clist_clear(GTK_CLIST(formats_list));
	GetTime();
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		/*
		 * Then a label with the format type.
		 */
		format_array[0]=format;
		gtk_clist_append(GTK_CLIST(formats_list), format_array);
	}
	active_item = saved_item;
	gtk_clist_select_row(GTK_CLIST(formats_list), saved_item, 0);
	gtk_clist_moveto(GTK_CLIST (formats_list), saved_item, -1, 0.0, 0.0);
	gtk_clist_thaw(GTK_CLIST(formats_list));
	gtk_signal_handler_unblock (GTK_OBJECT (formats_list), signal_id);

}


/*========================================================================
 *	Name:			ToggleAbbr
 *	Prototype:	void ToggleAbbr()
 *					
 *	Description:
 *	Close the Insert Dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleAbbr()
{
	int			i;

	if (use_abbr)
	{
		use_abbr = FALSE;
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[3])->child),XP_S_ABBR);
	}
	else
	{
		use_abbr = TRUE;
		gtk_label_set(GTK_LABEL(GTK_BIN(insert_buttons[3])->child),XP_S_FULL);
	}

	gtk_signal_handler_block (GTK_OBJECT (formats_list), signal_id);
	gtk_clist_freeze(GTK_CLIST(formats_list));
	gtk_clist_clear(GTK_CLIST(formats_list));

	GetTime();
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		/*
		 * Then a label with the format type.
		 */
		format_array[0]=format;
		gtk_clist_append(GTK_CLIST(formats_list), format_array);
	}
	active_item = saved_item;
	gtk_clist_select_row(GTK_CLIST(formats_list), saved_item, 0);
	gtk_clist_moveto(GTK_CLIST (formats_list), saved_item, -1, 0.0, 0.0);
	gtk_clist_thaw(GTK_CLIST(formats_list));
	gtk_signal_handler_unblock (GTK_OBJECT (formats_list), signal_id);

}


/*========================================================================
 *	Name:			FormatSelected 
 *	Prototype:	void FormatSelected
 *					
 *	Description:
 * Handle a left arrow click.
 *
 *	Notes:
 *========================================================================*/
static void
FormatSelected(
	GtkWidget		*clist,
	gint				row,
	gint				column,
	GdkEventButton	*event,
	gpointer			data
)
{
	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.
	 */
   saved_item = (int)row;
	active_item = saved_item;
}
