/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998,1999.
 *========================================================================
 *              FILE NAME: addr_manager.c
 *            DESCRIPTION: manage the Palm Pilot address list
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAddressManager XPUpdateAddrList
 *      PRIVATE FUNCTIONS: CloseAddrMgr UpdateDetails CategoryCB
 *                       : FilterFieldCB UpdateList CallXPEnvelope
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ADDR_MGR_C
#ifndef OLD_PILOT_ADDR

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void	CloseAddrMgr();
static void	UpdateDetails();
static void	FilterFieldCB();
static void	CategoryCB();
static void	UpdateList();
static void	CallXPEnvelope();


/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern Widget		Top_Level;

extern PilotAddr_T			*Pilot_Address;
extern PilotAddrCategory_T	*Pilot_Addr_Categories;

extern Pixmap details_xbm;


/* === Static Variables === */
static Widget	addr_mgr_dialog;
static char *filter_strings[]={
	"None",
	"Last",
	"First",
	"Company",
	"City",
	NULL
};

static int		filter_index;
static int		category_index;
static Widget	addr_list;
static Widget	search_text;
static Widget	details;

static ListEntries_T		*ListEntries=NULL;


/*========================================================================
 *	Name:			XPAddressManager
 *	Prototype:	void XPAddressManager()
 *					
 *	Description:
 * Setup and present the Address Manager window.
 *
 *	Notes:
 *========================================================================*/
void
XPAddressManager()
{
	char			buf[512];
	int			index;
	

	Widget		vbox, close_button; 
	Widget		envelope_button; 
	Widget		category_menu;
	Widget		filter_menu;
	Widget		frame1, frame2, label;
	XmString		xm_string;
	XmString		*category_xm_string;
	XmString		filter_xm_string[4];
	Arg			args[10];
	int			nargs;
	int			i;
	Dimension	width, height;
	
	PilotAddr_T				*current_link;
	PilotAddrCategory_T	*addr_cat_link;
	ListEntries_T			*current_list_entry, *next_list_entry;

	/*
	 * Grab PalmPilot addresses from text file.
	 */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);
	if ( XPPilotParseAddr(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCDB_FAIL);
		return;
	}

	/*
	 * Free up any existing link lists for the List entries.
	 * We'd have these if we've opened this dialog at least once.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * Allow only one window at a time.
	 */
	if ( addr_mgr_dialog != NULL )
		return;

	/*
	 * The addr dialog parent window.
	 */
	addr_mgr_dialog = 
		XPCreateGenericShell(AddrMgrDialog, ACCEPT_BUTTON|CLOSE_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)addr_mgr_dialog );
	vbox = XtNameToWidget(addr_mgr_dialog, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)addr_mgr_dialog);
	envelope_button = XtNameToWidget(addr_mgr_dialog, buf);
	sprintf(buf, "*GenericCancelB%d", (int)addr_mgr_dialog);
	close_button = XtNameToWidget(addr_mgr_dialog, buf);

	/*
	 * Change the Accept button to say "Envelopes"
	 */
	xm_string = XmStringCreateLtoR(XP_S_ENVELOPES, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( envelope_button, args, nargs );
	XmStringFree(xm_string);

	/*
	 * A scrolled list of address.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNvisibleItemCount, 18); nargs++;
   XtSetArg ( args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	addr_list = XmCreateScrolledList(
				vbox,
				"addrList",
				args,
				nargs);
	XtManageChild(addr_list);

	/*
	 * Category menu.
	 */
	index=0;
	addr_cat_link = Pilot_Addr_Categories;
	while ( addr_cat_link != NULL )
	{
		index++;
		addr_cat_link = addr_cat_link->next;
	}
	category_xm_string = (XmString *)XPSafeAlloc(sizeof(XmString)*(index+1));
	index=0;
	category_xm_string[index++] = XmStringCreateSimple(XP_S_ALL_ADDR);
	addr_cat_link = Pilot_Addr_Categories;
	while ( addr_cat_link != NULL )
	{
		category_xm_string[index++] = XmStringCreateSimple(addr_cat_link->name);
		addr_cat_link = addr_cat_link->next;
	}

	xm_string = XmStringCreateSimple(XP_S_PILOT_CATEGORIES);
	nargs = 0;
	XtSetArg ( args[nargs], XmNbuttons, category_xm_string); nargs++;
	XtSetArg ( args[nargs], XmNoptionLabel, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNbuttonSet, 0); nargs++;
	XtSetArg ( args[nargs], XmNbuttonCount, index); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, XtParent(addr_list)); nargs++;
   XtSetArg ( args[nargs], XmNsimpleCallback, CategoryCB); nargs++;
	category_menu = XmCreateSimpleOptionMenu(
				vbox,
				"option_menus",
				args, nargs);
	XmStringFree(xm_string);
	XtManageChild(category_menu);
	category_index=0;

	for ( i=0; i<index; i++ )
	{
		XmStringFree(category_xm_string[i]);
		i++;
	}
	free(category_xm_string);


	/*
	 * Filter On menu.
	 */
	index=0;
	while ( filter_strings[index] != NULL )
	{
		filter_xm_string[index] = XmStringCreateSimple(filter_strings[index]);
		index++;
	}

	xm_string = XmStringCreateSimple(XP_S_PILOT_FILTER);
	nargs = 0;
	XtSetArg ( args[nargs], XmNbuttons, filter_xm_string); nargs++;
	XtSetArg ( args[nargs], XmNoptionLabel, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNbuttonSet, 0); nargs++;
	XtSetArg ( args[nargs], XmNbuttonCount, index); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, XtParent(addr_list)); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, category_menu); nargs++;
   XtSetArg ( args[nargs], XmNsimpleCallback, FilterFieldCB); nargs++;
	filter_menu = XmCreateSimpleOptionMenu(
				vbox,
				"option_menus",
				args, nargs);
	XmStringFree(xm_string);
	XtManageChild(filter_menu);
	filter_index=0;


	/*
	 * Search prompt.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, XtParent(addr_list)); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, filter_menu); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); nargs++;
	frame1 = XmCreateFrame(
					vbox,
					"frame",
					args, nargs);
	xm_string = 
		XmStringCreateLtoR(XP_S_PILOT_FILTER_TEXT, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame1,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	XtManageChild(frame1);

	nargs=0;
	search_text = XmCreateTextField(
							frame1,
							"searchText",
							args, nargs
							);
	XtManageChild(search_text);
	XtSetSensitive(search_text, False);

	/* Add a tool tip */
	XtAddEventHandler(search_text,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_SEARCH);

	/* Add a tool tip */
	XtAddEventHandler(filter_menu,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_PILOT_FILTER);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		if (  ( strlen(current_link->last) != 0 ) &&
				( strlen(current_link->first) != 0 ) )
			sprintf(buf, "%s %s", current_link->first, current_link->last);
		else
		if ( strlen(current_link->last) != 0 )
			sprintf(buf, "%s", current_link->last);
		else
		if ( strlen(current_link->first) != 0 )
			sprintf(buf, "%s", current_link->first);
		else
		if ( strlen(current_link->company) != 0 )
			sprintf(buf, "%s", current_link->company);
		else
			sprintf(buf, "Unknown person?");

		xm_string = XmStringCreateSimple(buf);
		XmListAddItem(addr_list, xm_string, 0);
		XmStringFree(xm_string); 
		current_link = current_link->next;
	}

	/*
	 * Set the first entry in the list - if there is one - to be selected
	 */
	if ( Pilot_Address != NULL )
		XmListSelectPos(addr_list, 1, True);

	/*
	 * Details List.  This shows some relevant information about the
	 * selected entry in the list of names.  However, it won't show all the
	 * info.  That only comes when you edit the entry.
	 */
	nargs=0;
	XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg ( args[nargs], XmNbottomAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNleftWidget, XtParent(addr_list)); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, frame1); nargs++;
	XtSetArg ( args[nargs], XmNshadowType, XmSHADOW_ETCHED_OUT); nargs++;
	frame2 = XmCreateFrame(
					vbox,
					"frame",
					args, nargs);
	xm_string = 
		XmStringCreateLtoR(XP_S_PILOT_ADDR_DETAILS, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetArg ( args[nargs], XmNchildType, XmFRAME_TITLE_CHILD); nargs++;
	label= XmCreateLabel(
					frame2,
					"frame",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(label);
	XtManageChild(frame2);

	nargs=0;
	XtSetArg ( args[nargs], XmNbackgroundPixmap, details_xbm); nargs++;
	details= XmCreateLabel(
					frame2,
					"addresses",
					args, nargs);
	XmStringFree(xm_string);
	XtManageChild(details);

	/*
	 * Populate the list with all entries initially.
	 */
	UpdateList();
	UpdateDetails();

	/*
	 * Callback to use when an address has been selected.
	 */
	XtAddCallback( addr_list, XmNsingleSelectionCallback,
		UpdateDetails, (XtPointer)NULL);

	/*
	 * Callback to manage envelopes.
	 */
	XtAddCallback( envelope_button, XmNactivateCallback,
		CallXPEnvelope, (XtPointer)NULL);

	/*
	 * Callback to close the window.
	 */
	XtAddCallback( close_button, XmNactivateCallback,
		CloseAddrMgr, (XtPointer)NULL);

	/*
	 * Display the dialog.
	 */
	XtPopup(addr_mgr_dialog, XtGrabNone);
	XPCenterWindow(addr_mgr_dialog, NULL, addr_mgr_dialog);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( addr_mgr_dialog, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( addr_mgr_dialog, args, nargs); 

}


/*========================================================================
 *	Name:			CloseAddrMgr
 *	Prototype:	void CloseAddrMgr()
 *					
 *	Description:
 * Close the Address Manager dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseAddrMgr()
{
	if ( addr_mgr_dialog != NULL )
	{
		XtDestroyWidget(addr_mgr_dialog);
		addr_mgr_dialog = NULL;
	}

}


/*========================================================================
 *	Name:			XPUpdateAddrList
 *	Prototype:	void XPUpdateAddrList()
 *					
 *	Description:
 * Update the list when requested, like when a new category, filter field,
 * or search text is set.
 *
 *	Notes:
 *========================================================================*/
void
XPUpdateAddrList(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			CategoryCB
 *	Prototype:	static void CategoryCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
static void
CategoryCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	category_index = index;
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			FilterFieldCB
 *	Prototype:	static void FilterFieldCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
static void
FilterFieldCB(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	filter_index = index;
	if ( filter_index == 0 )
	{
		XmTextFieldSetString(search_text, "");
		XtSetSensitive(search_text, False);
	}
	else
		XtSetSensitive(search_text, True);

	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			UpdateDetails
 *	Prototype:	void UpdateDetails()
 *					
 *	Description:
 * Update the text in the Details box when a new entry is selected.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateDetails(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	PilotAddr_T				*current_link;
	ListEntries_T			*current_list_entry;
	XmString					xm_string;
	Arg						args[3];
	int						nargs;
	char						name[512];

	int	*selected_item, count;
	char	buf[512];

	/*
	 * Find out which list entry is selected.
	 */
	if ( XmListGetSelectedPos(addr_list, &selected_item, &count) == False )
	{
		xm_string = XmStringCreateLtoR(XP_S_NO_ADDRS, XmFONTLIST_DEFAULT_TAG);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
		XtSetValues( details, args, nargs );
		XmStringFree(xm_string);
	
		/* Update Envelope window, if its opened */
		XPUpdateToText(NULL, (PilotAddr_T *)NULL, NULL);
		return;
	}

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		if ( current_list_entry->id == *selected_item )
			break;
		current_list_entry = current_list_entry->next;
	}

	/*
	 * Place that links data into a buffer.
	 */
	if ( current_list_entry != NULL )
	{
		current_link = current_list_entry->linkptr;
		if (  (strlen(current_link->first)!=0) ||
				(strlen(current_link->last) !=0) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (strlen(current_link->company)!=0)
			sprintf(name, "%s", current_link->company);
		else
			sprintf(name, "Unknown person?");

		sprintf(buf,
			"%s\n"
			"%s\n%s %s %s\n"
			"%s\n%s\n%s\n%s\n%s",
			name,
			current_link->address, current_link->city, 
			current_link->state, current_link->zip, 
			current_link->home, 
			current_link->work, 
			current_link->fax, 
			current_link->other, 
			current_link->email 
		);

		/*
		 * Update the label in the Details box.
		 */
		xm_string = XmStringCreateLtoR(buf, XmFONTLIST_DEFAULT_TAG);
		nargs=0;
		XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
		XtSetValues( details, args, nargs );
		XmStringFree(xm_string);
	
		/* Update Envelope window, if its opened */
		XPUpdateToText(NULL, (PilotAddr_T *)current_link, NULL);
		XPFillEmailList((PilotAddr_T *)current_link);
	}
	else
		printf("No match between ListEntry links and selected item: %d\n",
			*selected_item);
}


/*========================================================================
 *	Name:			CallXPEnvelope
 *	Prototype:	void CallXPEnvelope()
 *					
 *	Description:
 * Grab the selected link and pass it to the Envelope dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CallXPEnvelope(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	ListEntries_T		*current_list_entry;
	int					*selected_item, count;

	/*
	 * Find out which list entry is selected.
	 */
	if ( XmListGetSelectedPos(addr_list, &selected_item, &count) == False )
	{
		XPEnvelope((PilotAddr_T *)NULL);
		return;
	}

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		if ( current_list_entry->id == *selected_item )
			break;
		current_list_entry = current_list_entry->next;
	}

	XPEnvelope((PilotAddr_T *)current_list_entry->linkptr);
}


/*========================================================================
 *	Name:			UpdateList
 *	Prototype:	void UpdateList()
 *					
 *	Description:
 * Update the scrolled list of names.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateList(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	Boolean					found1, found2;
	XmString					xm_string;
	char						buf[512];
	PilotAddr_T				*current_link;
	PilotAddrCategory_T	*cat_link;
	ListEntries_T			*current_list_entry=NULL;
	ListEntries_T			*next_list_entry=NULL;
	int						count;
	char						*search_string, *fielddata;
	int						num_entries;

	/*
	 * Clear the address list of all items.
	 */
	XtUnmanageChild(addr_list);
	XmListDeleteAllItems(addr_list);
	num_entries = 0;

	/*
	 * Free up current link list of List entries.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * Grab the current list options: category, filter field, filter string.
	 * Convert it to lowercase to make searching case insensitive.
	 */
	search_string = XmTextFieldGetString(search_text);
	XPConvertCase(search_string, XP_LOWER, 0);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		found1 = found2 = False;
		if ( strlen(search_string) > 0 )
		{
			switch (filter_index)
			{
				case 0:
					fielddata = NULL;
					break;
				case 1:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->last)+1);
					strcpy(fielddata, current_link->last);
					break;
				case 2:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->first)+1);
					strcpy(fielddata, current_link->first);
					break;
				case 3:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->company)+1);
					strcpy(fielddata, current_link->company);
					break;
				default:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->city)+1);
					strcpy(fielddata, current_link->city);
					break;
			}

			/*
			 * If the search string is not empty and this entry contains the
			 * string in the field specified, flag the entry.
			 */
			if ( fielddata != NULL )
			{
				XPConvertCase(fielddata, XP_LOWER, 0);
				if ( strstr(fielddata, search_string) != NULL )
					found1 = True;
				free(fielddata);
				fielddata = NULL;
			}
			else
				found1 = True;
		}
		else
			found1 = True;

		/*
		 * Now check if the category matches.
		 */
		if ( category_index != 0 )
		{
			cat_link = Pilot_Addr_Categories;
			count=1;
			while ( (cat_link != NULL ) && ( count != category_index ) )
			{
				cat_link = cat_link->next;
				count++;
			}
			if ( cat_link != NULL )
			{
				if ( strcmp(current_link->category, cat_link->name) == 0 )
				{
					found2 = True;
				}
			}
		}
		else
			found2 = True;

		/*
		 * If we found matches in both cases (or can ignore them) then we
		 * have a match.  Grab the appropriate fields to add to the list.
		 */
		if ( found1 && found2 )
		{
			if (  ( strlen(current_link->last) != 0 ) &&
					( strlen(current_link->first) != 0 ) )
				sprintf(buf, "%s %s", current_link->first, current_link->last);
			else
			if ( strlen(current_link->last) != 0 )
				sprintf(buf, "%s", current_link->last);
			else
			if ( strlen(current_link->first) != 0 )
				sprintf(buf, "%s", current_link->first);
			else
			if ( strlen(current_link->company) != 0 )
				sprintf(buf, "%s", current_link->company);
			else
				sprintf(buf, "Unknown person?");

			xm_string = XmStringCreateSimple(buf);
			XmListAddItem(addr_list, xm_string, 0);
			XmStringFree(xm_string); 
			num_entries++;

			/*
			 * Add entry to the link list that tracks whats visible.
			 */
			if ( ListEntries == NULL )
			{
				ListEntries = (ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				ListEntries->id = num_entries;
				ListEntries->linkptr = current_link;
				ListEntries->next = NULL;
				current_list_entry = ListEntries;
			}
			else
			{
				current_list_entry->next = 
					(ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				current_list_entry = current_list_entry->next;
				current_list_entry->id = num_entries;
				current_list_entry->linkptr = current_link;
				current_list_entry->next = NULL;
			}
		}

		current_link = current_link->next;
	}

	/*
	 * Free up temporary storage
	 */
	XtFree(search_string);

	if (num_entries > 0 )
		XmListSelectPos(addr_list, 1, True);

	XtManageChild(addr_list);
}
#endif
