/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: bindings.c
 *            DESCRIPTION: handle key bindings for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPActionAlarm XPActionAnchor XPActionDestroy
 *                       : XPActionHide XPActionErase XPActionCal
 *                       : XPActionSave XPActionPrint XPActionImport
 *                       : XPActionEnvelope XPActionProject
 *                       : XPActionFind XPActionName XPActionDate
 *                       : XPActionAddressManager
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_BINDINGS_C

/* === System Headers === */
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "motif.h"

/* === Project Headers === */
#include "XNotesPlus.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */
static StickyNote	*NoteFromWidget();

/* === Global Variables === */
Widget				Top_Level;
extern StickyNote	*notes;


/*========================================================================
 *	Name:			NoteFromWidget
 *	Prototype:	static StickyNote *NoteFromWidget()
 *					
 *	Description:
 * Find the corresponding note structure for the specified widget.
 *
 *	Notes:
 *		
 *========================================================================*/
static StickyNote *
NoteFromWidget(
	Widget		widget
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	for ( pn = notes; pn!= NULL; pn=pn->pn_next )
	{
		if (pn->pn_shellwidget == widget)
			return(pn);
		if (pn->pn_textwidget == widget)
			return(pn);
	}
	return(NULL);
}


/*========================================================================
 *	Name:			XPActionAlarm
 *	Prototype:	void XPActionAlarm()
 *					
 *	Description:
 *	Call Alarm window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionAlarm(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Toggle alarm status for note.
	 */
	if ( pn->pn_alarm )
		XPAlarmOff(widget, (XtPointer)pn->pn_index, NULL);
	else
		XPCreateTimerPrompt(widget, (XtPointer)pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionAnchor
 *	Prototype:	void XPActionAnchor()
 *					
 *	Description:
 *	Call Anchor window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionAnchor(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Toggle Anchor status for note.
	 */
	if ( pn->pn_anchor )
		XPUnAnchorNote(widget, (XtPointer)pn->pn_index, NULL);
	else
		XPAnchorNote(widget, (XtPointer)pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionDestroy
 *	Prototype:	void XPActionDestroy()
 *					
 *	Description:
 *	Call Destroy window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionDestroy(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 *  Destroy current note.
	 */
	XPDestroyNote(widget, pn, NULL);
}


/*========================================================================
 *	Name:			XPActionCal
 *	Prototype:	void XPActionCal()
 *					
 *	Description:
 *	Call Calendar window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionCal(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Insert a calendar into the current note.
	 */
	XPInsertCalendar(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionEnvelope
 *	Prototype:	void XPActionEnvelope()
 *					
 *	Description:
 *	Call Envelope window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionEnvelope(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Open Envelope dialog.
	 */
	XPEnvelope();
}


/*========================================================================
 *	Name:			XPActionProject
 *	Prototype:	void XPActionProject()
 *					
 *	Description:
 *	Call Project window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionProject(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Open Envelope dialog.
	 */
	XPProject();
}


/*========================================================================
 *	Name:			XPActionHide
 *	Prototype:	void XPActionHide()
 *					
 *	Description:
 *	Call Hide window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionHide(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Hide note.
	 */
	XPHideNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionErase
 *	Prototype:	void XPActionErase()
 *					
 *	Description:
 *	Call Erase window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionErase(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Erase note contents.
	 */
	XPEraseNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionPrint
 *	Prototype:	void XPActionPrint()
 *					
 *	Description:
 *	Call Print window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionPrint(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Print note contents.
	 */
	XPPrintNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionOpen
 *	Prototype:	void XPActionOpen()
 *					
 *	Description:
 *	Call Open window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionOpen(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Open file into note contents.
	 */
	XPOpenNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionSave
 *	Prototype:	void XPActionSave()
 *					
 *	Description:
 *	Call Save window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionSave(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Save note to file.
	 */
	XPSaveNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionFind
 *	Prototype:	void XPActionFind()
 *					
 *	Description:
 *	Call Find window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionFind(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Find the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Find text to file.
	 */
	XPFindTextPrompt(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionName
 *	Prototype:	void XPActionName()
 *					
 *	Description:
 *	Call Name window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionName(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Name the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Name note.
	 */
	XPNameNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionEmail
 *	Prototype:	void XPActionEmail()
 *					
 *	Description:
 *	Call Email window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionEmail(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Email the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Email note.
	 */
	XPCreateEmailNotePrompt(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionExport
 *	Prototype:	void XPActionExport()
 *					
 *	Description:
 *	Call Export window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionExport(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Export the corresponding note structure for the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Export note.
	 */
	XPExportNote(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionDate
 *	Prototype:	void XPActionDate()
 *					
 *	Description:
 *	Call Insert Date window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionDate(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	StickyNote		*pn;


	/*
	 * Insert the date in the corresponding note structure for 
	 * the invoking widget.
	 */
	if ( (pn = NoteFromWidget(widget)) == NULL )
	{
		XPErrPopUp("Can't find corresponding note structure.");
		return;
	}

	/*
	 * Insert date to note.
	 */
	XPInsertDates(widget, pn->pn_index, NULL);
}


/*========================================================================
 *	Name:			XPActionAddressManager
 *	Prototype:	void XPActionAddressManager()
 *					
 *	Description:
 *	Call Address Manager window feature.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPActionAddressManager(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Open the Address Manager dialog.
	 */
	XPAddressManager();
}


/*========================================================================
 *	Name:			XPSetNoteName
 *	Prototype:	void XPSetNoteName()
 *					
 *	Description:
 *	Call Save window feature for the invoking note.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPSetNoteName(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	Arg	args[2];
	int	nargs;
	int	index;

	/*
	 * Find the corresponding note structure for the invoking widget's
	 * userData field.
	 */
	nargs = 0;
   XtSetArg ( args[nargs], XmNuserData, &index); nargs++;
	XtGetValues(widget, args, nargs);
	
	/*
	 * Update Notes Name.
	 */
	XPUpdateName(widget, index, NULL);
}


/*========================================================================
 *	Name:			XPSendNote
 *	Prototype:	void XPSendNote
 *					
 *	Description:
 *	Send the note to the specified recipient when the user hits ENTER.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPSendNote(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	Arg	args[2];
	int	nargs;
	StickyNote	*pn;

	/*
	 * Find the corresponding note structure for the invoking widget's
	 * userData field.
	 */
	nargs = 0;
   XtSetArg ( args[nargs], XmNuserData, &pn); nargs++;
	XtGetValues(widget, args, nargs);
	
	/*
	 * Send the note to the recipient.
	 */
	XPEmailNote(widget, pn, NULL);
}


/*========================================================================
 *	Name:			XPChangeProject
 *	Prototype:	void XPChangeProject
 *					
 *	Description:
 *	Updates the project settings when ENTER is hit in the dialogs text field.
 *
 *	Notes:
 *		
 *========================================================================*/
void 
XPChangeProject(
	Widget		widget,
	XEvent		*event,
	String		*params,
	Cardinal		*num_params
)
{
	/*
	 * Send the note to the recipient.
	 */
	XPUpdateProject(NULL, NULL, NULL);
}

