/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: email.c
 *            DESCRIPTION: email routines for XNotesPlus notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPCreateEmailNotePrompt, XPEmailNote,
 *                       : XPEmailFillList
 *      PRIVATE FUNCTIONS: EmailPopDown
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES:
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_EMAIL_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>


#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPEmailNote();
void XPFillEmailList();


/* === Private routine prototypes === */
static void EmailPopDown();
static void EmailEntrySelected();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;
extern PilotAddr_T	*Pilot_Address;


/* === Static Variables === */
static Widget		email_widget=NULL;
static Widget		email_text;
#ifndef OLD_PILOT_ADDR
static Widget		email_list;
static Widget		email_name;
#endif



/*========================================================================
 *	Name:			XPCreateEmailNotePrompt
 *	Prototype:	XPCreateEmailNotePrompt(Widget widget, int index, 
 *						XtPointer call_data)
 *					
 *	Description:
 * Create a window in which the email address can be entered.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPCreateEmailNotePrompt(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote		*pn;
	Widget			email_label, email_cancel, email_accept;
	Widget			vbox;
	Arg				args[7];
	int				nargs;
	XmString			xm_string;
	char				buf[128];
	Dimension		width, height;

	/*
	 * Only one copy at at time!
	 */
	if ( email_widget != NULL )
		EmailPopDown();

	/*
	 * Find the note we're opening.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a popup shell widget
	 */
	email_widget = XPCreateGenericShell(NotesEmailPrompt, 
					ACCEPT_BUTTON | CANCEL_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)email_widget );
	vbox = XtNameToWidget(email_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)email_widget);
	email_cancel = XtNameToWidget(email_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)email_widget);
	email_accept = XtNameToWidget(email_widget, buf);


	/* the label window */
	xm_string = XmStringCreateLtoR(XP_S_EMAIL, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	email_label=XmCreateLabel(
			vbox,
			"label",
			args, nargs);
	XmStringFree(xm_string);
	XtManageChild(email_label);

	/*
	 * The text entry area for the email address.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, email_label); nargs++;
   XtSetArg ( args[nargs], XmNuserData, pn); nargs++;
	email_text = XmCreateTextField(
							vbox,
							"emailField",
							args, nargs
							);
	XtManageChild(email_text);
	if ( app_res.email_addr != NULL )
		XmTextFieldSetString(email_text, app_res.email_addr);

#ifndef OLD_PILOT_ADDR
	/*
	 * A label field for the name (to be filled in by the Address Manager.
	 */
	xm_string = XmStringCreateLtoR(XP_S_EMAIL_NONAME, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, email_text); nargs++;
   XtSetArg ( args[nargs], XmNuserData, pn); nargs++;
	email_name = XmCreateLabel(
							vbox,
							"emailName",
							args, nargs
							);
	XtManageChild(email_name);
	XmStringFree(xm_string);

	/*
	 * A scrolled list of email addresses taken from the Pilot
	 * List.
	 */
	nargs=0;
	XtSetArg(args[nargs], XmNvisibleItemCount, VISIBLE_EMAIL ); nargs++;
	XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg(args[nargs], XmNtopWidget, email_name); nargs++;
	XtSetArg(args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	email_list = XmCreateScrolledList(
							vbox,
							"scrolledList",
							args, nargs
							);
	XtManageChild(email_list);

	XtAddCallback( email_list, XmNsingleSelectionCallback, 
			EmailEntrySelected, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(email_list,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_EMAIL_LIST);
#endif


	/* 
	 * The Accept and Cancel button callbacks.
	 */
	XtAddCallback( email_accept, XmNactivateCallback, 
			XPEmailNote, (XtPointer)pn);
	XtAddCallback( email_cancel, XmNactivateCallback, 
			EmailPopDown, (XtPointer)NULL);

	/*
	 * Display the window.
	 */
	XtRealizeWidget(email_widget);
	XPCenterWindow(email_widget, pn->pn_shellwidget, email_widget);
	XtPopup( email_widget, XtGrabNone );

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( email_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( email_widget, args, nargs); 

}


/*========================================================================
 *	Name:			EmailPopDown
 *	Prototype:	EmailPopDown(Widget widget, XtPointer client_data, 
 *							XtPointer call_data)
 *					
 *	Description:
 * Close the Email window
 *========================================================================*/
static void
EmailPopDown(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	if ( email_widget != NULL )
	{
		XtDestroyWidget(email_widget);
		email_widget = NULL;
	}
}



/*========================================================================
 *	Name:			XPEmailNote
 *	Prototype:	XPEmailNote(Widget widget, StickyNote *pn, XtPointer call_data)
 *					
 *	Description:
 * Email a notes text to the specified recipient.
 *========================================================================*/
void
XPEmailNote(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	FILE			*fp;
	int			len;
	char			tmpfile[512];
	char			syscmd[1024];
	char			emailcmd[1024];
	char			*addressee;

	/*
	 * Don't email blank notes.
	 */
	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	if ( strlen(pn->pn_text) == 0 )
	{
		XPErrPopUp("No text in note to email!");
		return;
	}

	addressee = XmTextFieldGetString(email_text);
	if ( strlen(addressee) == 0 )
	{
		XPErrPopUp("You need to specify an address!");
		return;
	}

	/*
	 * Save the text to a temporary file
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	if ((fp = fopen(tmpfile, "w")) == NULL) 
	{
		fprintf(stderr, "xnotes: ");
		perror(tmpfile);
		sprintf(syscmd, XP_S_TEMP_OPEN_FAIL, tmpfile);
		XPErrPopUp(syscmd);
		return;
	}

	if ( pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = XmTextGetString(pn->pn_textwidget);
	len = strlen(pn->pn_text);
	fwrite(pn->pn_text, sizeof(char), len, fp);
	fclose(fp);

	/*
	 * Use the emailcmd resource to determine how to email the note
	 */
	sprintf (emailcmd, app_res.email_cmd, pn->pn_name, addressee);
	sprintf (syscmd, "cat %s | %s", tmpfile, emailcmd );
	system ( syscmd );

	/*
	 * clean up the temporary file
	 */
	sprintf( syscmd, "rm %s", tmpfile );
	system ( syscmd );

	EmailPopDown(NULL, NULL, NULL);
}

#ifndef OLD_PILOT_ADDR

/*========================================================================
 *	Name:			EmailEntrySelected
 *	Prototype:	EmailEntrySelected(Widget widget, XtPointer client_data, 
 *							XtPointer call_data)
 *					
 *	Description:
 * Email address selected by user from scrolled list.
 *========================================================================*/
static void
EmailEntrySelected(
	Widget		widget,
	XtPointer	client_data,
	XtPointer	call_data
)
{
	char	*choice;

	XmListCallbackStruct	*cbs = (XmListCallbackStruct *)call_data;

	XmStringGetLtoR(cbs->item, XmFONTLIST_DEFAULT_TAG, &choice);
	XmTextSetString(email_text, choice);
	XtFree(choice);
}


/*========================================================================
 *	Name:			XPFillEmailList
 *	Prototype:	XPFillEmailList(PilotAddr_T *current_link)
 *					
 *	Description:
 * Populate the list of email addresses.
 *========================================================================*/
void
XPFillEmailList(
	PilotAddr_T		*current_link
)
{
#ifdef DEBUG
	char *fname="XPFillEmailList()";
#endif
	XmString			xm_string;
	Arg				args[2];
	int				nargs;
	char				*buf, *bufptr;
	char				*entry = NULL;
	char				name[128];

	DBGEnter();

	/* Make sure the envelope dialog is available. */
	if ( email_widget == NULL )
		return;

	/*
	 * Remove any old entries.
	 */
	XmListDeleteAllItems(email_list);

	/*
	 * Build the name and stuff it in the label.
	 */
	if ( ( current_link->last != NULL ) && ( current_link->first != NULL ) )
	{
		if (	( strlen(current_link->last) != 0 ) && 
				( strlen(current_link->first) != 0 ) )
			sprintf(name, "%s %s", current_link->first, current_link->last);
		else
		if (	( strlen(current_link->first) == 0 ) && 
				( strlen(current_link->last) != 0 ) )
			sprintf(name, "%s", current_link->last);
		else
			sprintf(name, "%s", current_link->first);
	}
	else
	if ( current_link->last != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->first != NULL )
		sprintf(name, "%s", current_link->last);
	else
	if ( current_link->company != NULL )
		sprintf(name, "%s", current_link->company);

	DBGPrintf(DBG_INFO, ("name: *%s*\n", name));

	xm_string = XmStringCreateLtoR(name, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues ( email_name, args, nargs); 
	XmStringFree(xm_string);

	/*
	 * Pull out any email addresses.  Add them, with the name
	 * in parenthesis, if possible, to the ScrolledList window.
	 */

	/* Work field */
	if ( strstr(current_link->work, "@") != NULL )
	{
		if ( strstr(current_link->work, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->work)+1);
			strcpy(buf, current_link->work);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			entry = XPSafeAlloc(strlen(bufptr)+1);
			strcpy(entry, bufptr);
			xm_string = XmStringCreateSimple(bufptr);
			free(buf);
		}
		else
		{
			entry = XPSafeAlloc(strlen(current_link->work)+1);
			strcpy(entry, current_link->work);
			xm_string = XmStringCreateSimple(current_link->work);
		}
		XmListAddItem(email_list, xm_string, 0);
		XmStringFree(xm_string);
	}

	/* Home field */
	if ( strstr(current_link->home, "@") != NULL )
	{
		if ( strstr(current_link->home, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->home)+1);
			strcpy(buf, current_link->home);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(bufptr)+1);
				strcpy(entry, bufptr);
			}
			xm_string = XmStringCreateSimple(bufptr);
			free(buf);
		}
		else
		{
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(current_link->home)+1);
				strcpy(entry, current_link->home);
			}
			xm_string = XmStringCreateSimple(current_link->home);
		}
		XmListAddItem(email_list, xm_string, 0);
		XmStringFree(xm_string);
	}

	/* Fax field */
	if ( strstr(current_link->fax, "@") != NULL )
	{
		if ( strstr(current_link->fax, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->fax)+1);
			strcpy(buf, current_link->fax);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(bufptr)+1);
				strcpy(entry, bufptr);
			}
			xm_string = XmStringCreateSimple(bufptr);
			free(buf);
		}
		else
		{
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(current_link->fax)+1);
				strcpy(entry, current_link->fax);
			}
			xm_string = XmStringCreateSimple(current_link->fax);
		}
		XmListAddItem(email_list, xm_string, 0);
		XmStringFree(xm_string);
	}

	/* Email field */
	if ( strstr(current_link->email, "@") != NULL )
	{
		if ( strstr(current_link->email, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->email)+1);
			strcpy(buf, current_link->email);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(bufptr)+1);
				strcpy(entry, bufptr);
			}
			xm_string = XmStringCreateSimple(bufptr);
			free(buf);
		}
		else
		{
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(current_link->email)+1);
				strcpy(entry, current_link->email);
			}
			xm_string = XmStringCreateSimple(current_link->email);
		}
		XmListAddItem(email_list, xm_string, 0);
		XmStringFree(xm_string);
	}

	/* Other field */
	if ( strstr(current_link->other, "@") != NULL )
	{
		if ( strstr(current_link->other, ":") != NULL )
		{
			buf = XPSafeAlloc(strlen(current_link->other)+1);
			strcpy(buf, current_link->other);
			bufptr = strtok(buf, ":");
			bufptr = strtok(NULL, ":");
			while(strncmp(bufptr," ",1) == 0 )
				bufptr++;
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(bufptr)+1);
				strcpy(entry, bufptr);
			}
			xm_string = XmStringCreateSimple(bufptr);
			free(buf);
		}
		else
		{
			if ( entry == NULL )
			{
				entry = XPSafeAlloc(strlen(current_link->other)+1);
				strcpy(entry, current_link->other);
			}
			xm_string = XmStringCreateSimple(current_link->other);
		}
		XmListAddItem(email_list, xm_string, 0);
		XmStringFree(xm_string);
	}

	/*
	 * Set the first entry, if any.
	 */
	if ( entry != NULL )
	{
		XmTextSetString(email_text, entry);
		free(entry);
	}
	else
	{
		XmTextSetString(email_text, "");
	}
}
#endif
