/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: findtext.c
 *            DESCRIPTION: routines for finding text in a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPFindTextPrompt
 *      PRIVATE FUNCTIONS: FindPopDown
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_FINDTEXT_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void FindPopDown();
static void FindNext();
static void FindPrev();
static void FindHome();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;
extern Display			*display;


/* === Static Variables === */
static Widget				find_widget;
static Widget				find_text;
static XmTextPosition	home;
static Widget				radio_box;
static Widget				case_sensitive;
static Widget				case_insensitive;



/*========================================================================
 *	Name:			XPFindTextPrompt
 *	Prototype:	void XPFindTextPrompt(Widget widget, int index,
 *								XtPointer call_data)
 *					
 *	Description:
 * Create dialog for searching a note for text.
 *
 *	Input Arguments:
 *	int	index		notes index id
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPFindTextPrompt(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
#ifdef DEBUG
	char				*fname="XPFindTextPrompt";
#endif

	Arg				args[20];
	int				nargs;
	Widget			next_button, prev_button, home_button;
	Widget			cancel;
	Widget			vbox;
	StickyNote		*pn;
	char				buf[256];
	XmString			xm_string;
	Dimension		width, height;

	DBGEnter();

	/*
	 * Allow only one copy at a time.
	 */
	if ( find_widget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Create a dialog and set its title.
	 */
	find_widget = XPCreateGenericShell(FindTextDialog, 
				CANCEL_BUTTON | OVERWRITE_BUTTON | APPEND_BUTTON | HELP_BUTTON );

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)find_widget );
	vbox = XtNameToWidget(find_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)find_widget);
	cancel = XtNameToWidget(find_widget, buf);
 
	/*
	 * Rename the other buttons, then give them callbacks.
	 */
	sprintf(buf, "*GenericOverwriteB%d", (int)find_widget);
	next_button = XtNameToWidget(find_widget, buf);
	xm_string = XmStringCreateSimple(XP_S_NEXT);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( next_button, args, nargs );
	XmStringFree(xm_string);

	sprintf(buf, "*GenericAppendB%d", (int)find_widget);
	prev_button = XtNameToWidget(find_widget, buf);
	xm_string = XmStringCreateSimple(XP_S_PREV);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( prev_button, args, nargs );
	XmStringFree(xm_string);

	sprintf(buf, "*GenericHelpB%d", (int)find_widget);
	home_button = XtNameToWidget(find_widget, buf);
	xm_string = XmStringCreateSimple(XP_S_HOME);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( home_button, args, nargs );
	XmStringFree(xm_string);

	/*
	 * Callbacks for Next/Prev/Home.
	 */
	XtAddCallback( next_button, XmNactivateCallback, FindNext, (XtPointer)pn);
	XtAddCallback( prev_button, XmNactivateCallback, FindPrev, (XtPointer)pn);
	XtAddCallback( home_button, XmNactivateCallback, FindHome, (XtPointer)pn);

	/*
	 * A text field to input search string.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_FORM); nargs++;
	find_text = XmCreateTextField(
				vbox,
				"find_text",
				args, nargs);
	XtManageChild(find_text);
	XmTextFieldSetMaxLength(find_text, 128);

	/*
	 * Radio buttons for setting case for search.
	 */
	nargs=0;
   XtSetArg ( args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
   XtSetArg ( args[nargs], XmNtopAttachment, XmATTACH_WIDGET); nargs++;
   XtSetArg ( args[nargs], XmNtopWidget, find_text); nargs++;
	XtSetArg ( args[nargs], XmNorientation, XmHORIZONTAL); nargs++;
	radio_box = XmCreateRadioBox(
					vbox,
					"toggles",
					args, nargs);

	xm_string = 
		XmStringCreateLtoR(XP_S_CASE_INSENSITIVE, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	case_insensitive = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(case_insensitive);
	XmStringFree(xm_string);

	xm_string = XmStringCreateLtoR(XP_S_CASE_SENSITIVE, XmFONTLIST_DEFAULT_TAG);
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	case_sensitive = XmCreateToggleButton(
						radio_box,
						"toggleButton",
						args, nargs);
	XtManageChild(case_sensitive);
	XmStringFree(xm_string);
	XtManageChild(radio_box);

	/* Make insensitive the default search type */
	XmToggleButtonSetState(case_insensitive, True, False);

	/*
	 * Cancel button callback.
	 */
	XtAddCallback( cancel, XmNactivateCallback, 
		FindPopDown, (XtPointer)find_widget);

	/*
	 * Set the current location as the "Home" location.
	 */
	home = XmTextGetInsertionPosition(pn->pn_textwidget);

	/*
	 * Stick the dialog in the middle of the note.
	 */
	XtRealizeWidget(find_widget);
	XPCenterWindow(find_widget, pn->pn_shellwidget, find_widget);
	XtPopup(find_widget, XtGrabNone);

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( find_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( find_widget, args, nargs); 
}


/*========================================================================
 *	Name:			FindPopDown 
 *					
 *	Description:
 * FindPopDown - Destroy the Find window 
 *
 *========================================================================*/
static void
FindPopDown()
{
	if ( find_widget != NULL )
	{
		XtDestroyWidget(find_widget);
		find_widget = NULL;
	}

}


/*========================================================================
 *	Name:			FindNext
 *					
 *	Description:
 * Finds the next occurance of the specified string, if any, in the note.
 *
 * Notes:
 * Due to a bug in LessTif's XmTextFindString(), I can't use that routine.
 * I had to write my own little routine, but its not that big a deal.
 *========================================================================*/
static void
FindNext(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	char					*string;
	char					*string2;
	char					buf[256];
	XmTextPosition		position;
	XmTextPosition		curr_pos;
	int					sensitive;

	/*
	 * Get text string to search for.
	 */
	string = XmTextFieldGetString(find_text);
	string2 = XmTextGetString(pn->pn_textwidget);

	if ( strlen(string) == 0 )
	{
		XtFree(string);
		XtFree(string2);
		return;
	}

	/*
	 * If doing a case insensitive search, convert both strings to the same
	 * case.
	 */
	if ( XmToggleButtonGetState(case_insensitive) == True )
		sensitive = (int)False;
	else
		sensitive = (int)True;

	curr_pos = XmTextGetInsertionPosition(pn->pn_textwidget)+1;

	/*
	 * Find the text.
	 */
	position = 
		(XmTextPosition) XPFindString(XP_FORWARD, sensitive, curr_pos, 
			string, string2, pn);

	/*
	 * Update current position.
	 */
	if ( position != -1 )
	{
		curr_pos = position;
		XmTextSetSelection(pn->pn_textwidget, curr_pos, 
			curr_pos+strlen(string), XtLastTimestampProcessed(display));
		XmTextShowPosition(pn->pn_textwidget, curr_pos);
		XmTextSetInsertionPosition(pn->pn_textwidget,curr_pos);

	}
	else
	{
		sprintf(buf, "%s: %s", XP_S_FIND_FAIL, string);
		XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
	}

	XtFree(string);
	XtFree(string2);
}


/*========================================================================
 *	Name:			FindPrev 
 *					
 *	Description:
 * FindPrev - Finde previous occurance, if any, of string in text widget.
 *
 * Notes:
 * Converted this to use generic XPFindString() since it makes it easier to
 * deal with. 
 *========================================================================*/
static void
FindPrev(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	char					*string;
	char					*string2;
	char					buf[256];
	XmTextPosition		position;
	XmTextPosition		curr_pos;
	int					sensitive;

	/*
	 * Get text string to search for.
	 */
	string = XmTextFieldGetString(find_text);
	string2 = XmTextGetString(pn->pn_textwidget);

	if ( strlen(string) == 0 )
	{
		XtFree(string);
		XtFree(string2);
		return;
	}

	/*
	 * If doing a case insensitive search, convert both strings to the same
	 * case.
	 */
	if ( XmToggleButtonGetState(case_insensitive) == True )
		sensitive = (int)False;
	else
		sensitive = (int)True;

	curr_pos = XmTextGetInsertionPosition(pn->pn_textwidget)-1;

	/*
	 * Find the text.
	 */
	position = 
		(XmTextPosition) XPFindString(XP_BACKWARD, sensitive, curr_pos, 
			string, string2, pn);

	/*
	 * Update current position.
	 */
	if ( position != -1 )
	{
		curr_pos = position;
		XmTextSetSelection(pn->pn_textwidget, curr_pos, 
			curr_pos+strlen(string), XtLastTimestampProcessed(display));
		XmTextShowPosition(pn->pn_textwidget, curr_pos);
		XmTextSetInsertionPosition(pn->pn_textwidget,curr_pos);

	}
	else
	{
		sprintf(buf, "%s: %s", XP_S_FIND_FAIL, string);
		XPMsgWindow(buf, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK,
			NULL, NULL, NULL, NULL, NULL, NULL);
	}

	XtFree(string);
	XtFree(string2);

}


/*========================================================================
 *	Name:			FindHome 
 *					
 *	Description:
 * FindHome - Jump back to original locationin the text widget.
 *
 *========================================================================*/
static void
FindHome(
	Widget		widget,
	StickyNote	*pn,
	XtPointer	call_data
)
{
	XmTextSetInsertionPosition(pn->pn_textwidget, home);
}

