/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995,1996,1997,1998.
 *========================================================================
 *              FILE NAME: inserts.c
 *            DESCRIPTION: routines that insert simple text data into notes
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPInsertCalendar, XPInsertDates
 *      PRIVATE FUNCTIONS: CreateInsertPrompt, InsertAccept, InsertPopDown,
 *								 : InsertLeftArrowCB, 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_INSERTS_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#include "motif.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void InsertPopDown();
static void InsertAccept();
static void FormatSelected();
static void CreateInsertPrompt();
static void GetTime();
static void SetFormat();
static void ToggleGMT();
static void ToggleAbbr();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern Widget			Top_Level;


/* === Static Variables === */
static Widget 		insert_widget=NULL;
static Widget		insert_buttons[4];
static Widget		format_list;
static int			active_item=1;
static int			saved_item=-1;

static char			dayname[12];
static char			year[5];
static char			monthname[12];
static int			daynum;
static int			wdnum;
static int			monthnum;
static int			yearnum;
static int			hournum;
static int			minnum;
static Boolean		use_gmtime=False;
static Boolean		use_abbr=False;
static char			format[32];


/*========================================================================
 *	Name:			XPInsertCalendar
 *	Prototype:	void XPInsertCalendar(Widget widget, int index, 
 *							XtPointer call_data)
 *					
 *	Description:
 * Insert this months calendar into the note
 *
 *	Notes:
 *========================================================================*/
void
XPInsertCalendar(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote	*pn;
	char			tmpfile[512];
	char			cmd[1024];
	int			rc;

	/*
	 * Find the note we're adding the calender to.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Run the calendar generating program to get the text.
	 * We use system() instead of popen() because the latter
	 * doesn't reliably tell us if the process actually ran.
	 * system() will provide the return value of the invoked
	 * program.
	 */
	sprintf(tmpfile,"%s/xp%d", app_res.tmp_dir, (int)getpid());
	sprintf (cmd, "%s > %s", app_res.calendar_cmd, tmpfile );
	if ( (rc=system ( cmd )) != 0 ) 
	{

		sprintf(cmd, XP_S_CANT_RUN_CMD, app_res.calendar_cmd, rc);
		XPErrPopUp(cmd);

		/* cleanup, if necessary */
		sprintf (cmd, "rm %s", tmpfile );
		system ( cmd );
		return;
	}

	/* stuff the output into the note */
	XPInsertText(pn, tmpfile, TRUE, FILE_TYPE);

	/*
	 * Changes occured to the note - set its save widget sensitive.
	 */
	XtSetSensitive ( pn->pn_savewidget, True );
}



/*========================================================================
 *	Name:			XPInsertDates
 *	Prototype:	void XPInsertDates(Widget widget, int index, XtPointer call_data)
 *					
 *	Description:
 * Callback for inserting a formatted date into the note.
 *
 *	Notes:
 *========================================================================*/
void
XPInsertDates(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	StickyNote 	*pn;

	/*
	 * Find the note we're hiding.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	CreateInsertPrompt(pn);

}


/*========================================================================
 *	Name:			CreateInsertPrompt
 *	Prototype:	static void CreateInsertPrompt(StickyNote *pn)
 *					
 *	Description:
 * Create a prompt for insert information
 *
 *	Notes:
 *========================================================================*/
static void
CreateInsertPrompt(
	StickyNote	*pn
)
{
	Widget			vbox;
	int				i;
	char				buf[256];
	XmString			xm_string;
	Arg				args[10];
	int				nargs;
	Dimension		width, height;

	/*
	 * Prevent multiple windows from being opened at once.
	 */
	if ( insert_widget != NULL )
		InsertPopDown();

	/*
	 * Create a popup shell widget
	 */
	insert_widget = 
		XPCreateGenericShell(NotesInsertPrompt, 
			ACCEPT_BUTTON | CANCEL_BUTTON | APPEND_BUTTON | OVERWRITE_BUTTON);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	sprintf(buf, "*GenericForm%d", (int)insert_widget );
	vbox = XtNameToWidget(insert_widget, buf);
	sprintf(buf, "*GenericAcceptB%d", (int)insert_widget);
	insert_buttons[0] = XtNameToWidget(insert_widget, buf);
	sprintf(buf, "*GenericCancelB%d", (int)insert_widget);
	insert_buttons[1] = XtNameToWidget(insert_widget, buf);

	/*
	 * Buttons for GMT, Abbreviated Weekday names.
	 */
	sprintf(buf, "*GenericAppendB%d", (int)insert_widget);
	insert_buttons[2] = XtNameToWidget(insert_widget, buf);
	sprintf(buf, "*GenericOverwriteB%d", (int)insert_widget);
	insert_buttons[3] = XtNameToWidget(insert_widget, buf);

	if (use_gmtime)
	{
		xm_string = XmStringCreateLtoR(XP_S_LOCAL, XmFONTLIST_DEFAULT_TAG);
	}
	else
	{
		xm_string = XmStringCreateLtoR(XP_S_GMT, XmFONTLIST_DEFAULT_TAG);
	}
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( insert_buttons[2], args, nargs );
	XmStringFree(xm_string);

	if (use_abbr)
	{
		xm_string = XmStringCreateLtoR(XP_S_FULL, XmFONTLIST_DEFAULT_TAG);
	}
	else
	{
		xm_string = XmStringCreateLtoR(XP_S_ABBR, XmFONTLIST_DEFAULT_TAG);
	}
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( insert_buttons[3], args, nargs );
	XmStringFree(xm_string);

	/*
	 * A scrolled list of time/date formats.
	 */
	nargs=0;
	XtSetArg(args[nargs], XmNvisibleItemCount, VISIBLE_FORMATS ); nargs++;
	XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); nargs++;
	XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); nargs++;
	XtSetArg(args[nargs], XmNselectionPolicy, XmSINGLE_SELECT); nargs++;
	format_list = XmCreateScrolledList(
							vbox,
							"scrolledList",
							args, nargs
							);
	XtManageChild(format_list);
	XtAddCallback( format_list, XmNsingleSelectionCallback, 
			FormatSelected, (XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(format_list,
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_INSERT_FORMAT);

	GetTime();
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		/*
		 * Then a label with the format type.
		 */
		xm_string = XmStringCreateSimple(format);
		XmListAddItem(format_list, xm_string, 0);
		XmStringFree(xm_string);
	}
	if (saved_item != -1 )
		active_item = saved_item - 1;
	else
	{
		saved_item = 1;
		active_item = 0;
	}
	XmListSelectPos(format_list, saved_item, FALSE);
	if ( (NUM_FORMATS - saved_item) < VISIBLE_FORMATS )
		XmListSetBottomPos(format_list, saved_item);
	else
		XmListSetPos(format_list, saved_item);

	/*
	 * create the Cancel/Accept command buttons, plus a horizontal
	 * box for them.
	 */
	XtAddCallback( insert_buttons[0], XmNactivateCallback, InsertAccept, 
			(XtPointer)pn);
	XtAddCallback( insert_buttons[1], XmNactivateCallback, InsertPopDown, 
			(XtPointer)NULL);
	XtAddCallback( insert_buttons[2], XmNactivateCallback, ToggleGMT, 
			(XtPointer)NULL);
	XtAddCallback( insert_buttons[3], XmNactivateCallback, ToggleAbbr, 
			(XtPointer)NULL);

	/* Add a tool tip */
	XtAddEventHandler(insert_buttons[2],
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_INSERT_GMT);

	/* Add a tool tip */
	XtAddEventHandler(insert_buttons[3],
		EnterWindowMask | LeaveWindowMask | ButtonPressMask,
		False, (XtEventHandler)XPToolTips,
		(XtPointer)XP_TTID_INSERT_ABBR);

	/*
	 * Pop up the window.
	 */
	XtRealizeWidget(insert_widget);
	XPCenterWindow(insert_widget, pn->pn_shellwidget, insert_widget);
	XtPopup( insert_widget, XtGrabNone );

	/*
	 * This bit of code forces the pop up window to stay its
	 * initial size, despite the users attempts to resize it.
	 * By doing this, I avoid having to muck extensively with
	 * aligning objects after a resize.
	 */
	nargs = 0;
	XtSetArg ( args[nargs], XmNwidth, &width); nargs++;
	XtSetArg ( args[nargs], XmNheight, &height); nargs++;
	XtGetValues ( insert_widget, args, nargs);
	nargs = 0;
	XtSetArg ( args[nargs], XmNmaxWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNmaxHeight, height); nargs++;
	XtSetArg ( args[nargs], XmNminWidth, width); nargs++;
	XtSetArg ( args[nargs], XmNminHeight, height); nargs++;
	XtSetValues ( insert_widget, args, nargs); 

}


/*========================================================================
 *	Name:			InsertAccept
 *	Prototype:	void InsertAccept(Widget widget, StickyNote *pn, 
 *								XtPointer call_data)
 *					
 *	Description:
 * Accept selected date value. This routine has the potential to be 
 * very OS specific.
 *
 *	Notes:
 *========================================================================*/
void
InsertAccept(
	Widget		widget,
	StickyNote 	*pn,
	XtPointer	call_data
)
{
	GetTime();
	SetFormat();

	/* stuff the output into the note */
	XPInsertText(pn, format, FALSE, TEXT_TYPE);

	XtSetSensitive ( pn->pn_savewidget, True );
	InsertPopDown();
}


/*========================================================================
 *	Name:			GetTime 
 *	Prototype:	void GetTime()
 *					
 *	Description:
 *	Grab the current date fields for display in the dialog.
 *
 *	Notes:
 *========================================================================*/
static void
GetTime()
{
	time_t		time_tp;
	struct tm	*tm_tp;

	/* get todays date and current time */
	time(&time_tp);
	if ( use_gmtime )
		tm_tp = gmtime(&time_tp);
	else
		tm_tp = localtime(&time_tp);

	/*
	 * increment the numeric month, since localtime()/gmtime()
	 * return the month from 0 to 11.
	 */
	tm_tp->tm_mon++;

	/*
	 * Save integer values.
	 */
	daynum   = tm_tp->tm_mday;
	monthnum = tm_tp->tm_mon;
	yearnum  = tm_tp->tm_year;
	hournum  = tm_tp->tm_hour;
	minnum   = tm_tp->tm_min;
	wdnum    = tm_tp->tm_wday;
}



/*========================================================================
 *	Name:			SetFormat() 
 *	Prototype:	void SetFormat()
 *					
 *	Description:
 *	Format the current time into the format buffer.
 *
 *	Notes:
 *========================================================================*/
static void
SetFormat()
{
	/*
	 * convert numeric values to strings now, even if we don't use them,
	 * since we need these in more than one place.
	 */
	if (use_abbr)
	{
		switch ( wdnum )
		{
			case 0:	sprintf(dayname, "Sun"); break;
			case 1:	sprintf(dayname, "Mon"); break;
			case 2:	sprintf(dayname, "Tue"); break;
			case 3:	sprintf(dayname, "Wed"); break;
			case 4:	sprintf(dayname, "Thu"); break;
			case 5:	sprintf(dayname, "Fri"); break;
			case 6:	sprintf(dayname, "Sat"); break;
		}
		switch ( monthnum )
		{
			case  0: sprintf(monthname, "Jan"); break;
			case  1: sprintf(monthname, "Feb"); break;
			case  2: sprintf(monthname, "Mar"); break;
			case  3: sprintf(monthname, "Apr"); break;
			case  4: sprintf(monthname, "May"); break;
			case  5: sprintf(monthname, "Jun"); break;
			case  6: sprintf(monthname, "Jul"); break;
			case  7: sprintf(monthname, "Aug"); break;
			case  8: sprintf(monthname, "Sep"); break;
			case  9: sprintf(monthname, "Oct"); break;
			case 10: sprintf(monthname, "Nov"); break;
			case 11: sprintf(monthname, "Dec"); break;
		}

	}
	else
	{
		switch ( wdnum )
		{
			case 0:	sprintf(dayname, "Sunday"); break;
			case 1:	sprintf(dayname, "Monday"); break;
			case 2:	sprintf(dayname, "Tuesday"); break;
			case 3:	sprintf(dayname, "Wednesday"); break;
			case 4:	sprintf(dayname, "Thursday"); break;
			case 5:	sprintf(dayname, "Friday"); break;
			case 6:	sprintf(dayname, "Saturday"); break;
		}
		switch ( monthnum )
		{
			case  0: sprintf(monthname, "January"); break;
			case  1: sprintf(monthname, "February"); break;
			case  2: sprintf(monthname, "March"); break;
			case  3: sprintf(monthname, "April"); break;
			case  4: sprintf(monthname, "May"); break;
			case  5: sprintf(monthname, "June"); break;
			case  6: sprintf(monthname, "July"); break;
			case  7: sprintf(monthname, "August"); break;
			case  8: sprintf(monthname, "September"); break;
			case  9: sprintf(monthname, "October"); break;
			case 10: sprintf(monthname, "November"); break;
			case 11: sprintf(monthname, "December"); break;
		}

	}

	if ( yearnum > 100 )
		sprintf(year,"20");
	else
		sprintf(year,"19");

	switch (active_item)
	{
		case 0:
			sprintf (format, 
				"%02d/%02d/%02d", monthnum, daynum, yearnum );
			break;

		case 1:
			sprintf (format, 
				"<%02d/%02d/%02d>", monthnum, daynum, yearnum );
			break;

		case 2:
			sprintf (format, 
				"(%02d/%02d/%02d)", monthnum, daynum, yearnum );
			break;

		case 3:
			sprintf (format, 
				"%02d/%02d/%02d - %02d:%02d", 
				monthnum, daynum, yearnum, hournum, minnum );
			break;

		case 4:
			sprintf (format, 
				"%02d/%02d/%02d(%02d:%02d)", 
				monthnum, daynum, yearnum, hournum, minnum );
			break;

		case 5:
			sprintf (format, 
				"%s %d, %s%d", monthname, daynum, year, yearnum);
			break;

		case 6:
			sprintf (format, 
				"%s %d, %s%d - %02d:%02d", 
				monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 7:
			sprintf (format, 
				"%s %d, %s%d (%02d:%02d)", 
				monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 8:
			sprintf (format, 
				"%s %s %d, %s%d", 
				dayname, monthname, daynum, year, yearnum);
			break;

		case 9:
			sprintf (format, 
				"%s %s %d, %s%d - %02d:%02d", 
				dayname, monthname, daynum, year, yearnum, hournum, minnum );
			break;

		case 10:
			sprintf (format, 
				"%s %s %d, %s%d (%02d:%02d)", 
				dayname, monthname, daynum, year, yearnum, hournum, minnum );
			break;

		/* International formats */
		case 11:
			sprintf (format, 
				"%02d/%02d/%02d", daynum, monthnum, yearnum );
			break;

		case 12:
			sprintf (format, 
				"<%02d/%02d/%02d>", daynum, monthnum, yearnum );
			break;

		case 13:
			sprintf (format, 
				"(%02d/%02d/%02d)", daynum, monthnum, yearnum );
			break;

		case 14:
			sprintf (format, 
				"%02d/%02d/%02d - %02d:%02d", 
				daynum, monthnum, yearnum, hournum, minnum );
			break;

		case 15:
			sprintf (format, 
				"%02d/%02d/%02d(%02d:%02d)", 
				daynum, monthnum, yearnum, hournum, minnum );
			break;

		case 16:
			sprintf (format, 
				"%d %s, %s%d", daynum, monthname, year, yearnum);
			break;

		case 17:
			sprintf (format, 
				"%d %s, %s%d - %02d:%02d", 
				daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 18:
			sprintf (format, 
				"%d %s, %s%d (%02d:%02d)", 
				daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 19:
			sprintf (format, 
				"%s %d %s, %s%d", 
				dayname, daynum, monthname, year, yearnum);
			break;

		case 20:
			sprintf (format, 
				"%s %d %s, %s%d - %02d:%02d", 
				dayname, daynum, monthname, year, yearnum, hournum, minnum );
			break;

		case 21:
			sprintf (format, 
				"%s %d %s, %s%d (%02d:%02d)", 
				dayname, daynum, monthname, year, yearnum, hournum, minnum );
			break;
	}
}


/*========================================================================
 *	Name:			InsertPopDown
 *	Prototype:	void InsertPopDown()
 *					
 *	Description:
 *	Close the Insert Dialog.
 *
 *	Notes:
 *========================================================================*/
static void
InsertPopDown()
{
	if ( insert_widget != NULL )
	{
		XtDestroyWidget( insert_widget );
		insert_widget = NULL;
	}
}



/*========================================================================
 *	Name:			FormatSelected 
 *	Prototype:	void FormatSelected
 *					
 *	Description:
 * Handle a left arrow click.
 *
 *	Notes:
 *========================================================================*/
static void
FormatSelected(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	XmListCallbackStruct *cbs = (XmListCallbackStruct *)call_data;

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.
	 */
   saved_item = cbs->item_position;
	active_item = saved_item - 1;
}


/*========================================================================
 *	Name:			ToggleGMT 
 *	Prototype:	void ToggleGMT
 *					
 *	Description:
 * Handle a left arrow click.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleGMT(
	Widget		widget,
	int			index,
	XtPointer	call_data
)
{
	XmString		xm_string;
	Arg			args[2];
	int			nargs;
	int			i;

	if (use_gmtime)
	{
		use_gmtime = False;
		xm_string = XmStringCreateLtoR(XP_S_GMT, XmFONTLIST_DEFAULT_TAG);
	}
	else
	{
		use_gmtime = True;
		xm_string = XmStringCreateLtoR(XP_S_LOCAL, XmFONTLIST_DEFAULT_TAG);
	}
	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( insert_buttons[2], args, nargs );
	XmStringFree(xm_string);

	XtUnmanageChild(format_list);
	XmListDeleteAllItems(format_list);
	GetTime();
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		/*
		 * Then a label with the format type.
		 */
		xm_string = XmStringCreateSimple(format);
		XmListAddItem(format_list, xm_string, 0);
		XmStringFree(xm_string);
	}
	active_item = saved_item - 1;
	XmListSelectPos(format_list, saved_item, FALSE);
	if ( (NUM_FORMATS - saved_item) < VISIBLE_FORMATS )
		XmListSetBottomPos(format_list, saved_item);
	else
		XmListSetPos(format_list, saved_item);
	XtManageChild(format_list);
}


/*========================================================================
 *	Name:			ToggleAbbr
 *	Prototype:	void ToggleAbbr()
 *					
 *	Description:
 *	Close the Insert Dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ToggleAbbr()
{
	XmString		xm_string;
	Arg			args[2];
	int			nargs;
	int			i;
	if (use_abbr)
	{
		use_abbr = False;
		xm_string = XmStringCreateLtoR(XP_S_ABBR, XmFONTLIST_DEFAULT_TAG);
	}
	else
	{
		use_abbr = True;
		xm_string = XmStringCreateLtoR(XP_S_FULL, XmFONTLIST_DEFAULT_TAG);
	}

	nargs=0;
	XtSetArg ( args[nargs], XmNlabelString, xm_string); nargs++;
	XtSetValues( insert_buttons[3], args, nargs );
	XmStringFree(xm_string);

	XtUnmanageChild(format_list);
	XmListDeleteAllItems(format_list);
	GetTime();
	for ( i=0; i<NUM_FORMATS; i++)
	{
		active_item = i;
		SetFormat();

		/*
		 * Then a label with the format type.
		 */
		xm_string = XmStringCreateSimple(format);
		XmListAddItem(format_list, xm_string, 0);
		XmStringFree(xm_string);
	}
	active_item = saved_item - 1;
	XmListSelectPos(format_list, saved_item, FALSE);
	if ( (NUM_FORMATS - saved_item) < VISIBLE_FORMATS )
		XmListSetBottomPos(format_list, saved_item);
	else
		XmListSetPos(format_list, saved_item);
	XtManageChild(format_list);
}
